#include "CircleGridMap.h"
#include "DLAElement.h"
#include <string>
#include <sstream>
#include <iostream>
#include <cmath>


CircleGridMap::CircleGridMap(int numOfRows, int numOfCols, int xUnits, int yUnits) throw(bad_alloc)
  :m_numOfRows(numOfRows), m_numOfCols(numOfCols), m_xUnits(xUnits), m_yUnits(yUnits)
  ,m_halfRows(numOfRows/2), m_halfCols(numOfCols/2), m_xHalfUnits(xUnits/2), m_yHalfUnits(yUnits/2), grid(NULL)
  ,m_xMin(-numOfRows/2), m_xMax(numOfRows/2-1), m_yMin(-numOfRows/2), m_yMax(numOfRows/2-1)
{
	//Because the coordinate of [0,0] is included, so one more row
	//and one more column are needed to accommodate the 0th row and 0th column.
	int i, j;
	grid = new char*[m_numOfRows];
	try{
	  for(i = 0; i < m_numOfRows; i++){
		grid[i] = new char[numOfCols];
	  }
	}
	catch(...){
		for(j=0; j<i; j++){
			delete[] grid[j];
		}
		delete[] grid;
		//grid = NULL;
		throw bad_alloc();
	}
	reset();
}

void CircleGridMap::reset(){
	int i, j;
	for( i=m_xMin; i<m_xMax; i++){
		for( j=m_yMin; j<m_yMax; j++){
			(*this)(i, j)='0';
		}
	}
}

CircleGridMap::CircleGridMap(const CircleGridMap& src) throw(bad_alloc){
	try{
	copyFrom(src);
	}
	catch(...){
		throw bad_alloc();
	}
}

CircleGridMap& CircleGridMap::operator=(const CircleGridMap& rhs) throw(bad_alloc){	
	//Check for self-assignment.
	if(this==&rhs){
		return (*this);
	}
	
	//Free the old memory.
	int i;
	for(i=0; i<m_numOfRows; i++){
		delete[] grid[i];
	}
	delete[] grid;
	
	//copy the new memory
	try{
	  copyFrom(rhs);
	}
	catch(...){
		throw bad_alloc();
	}
	return (*this);
}

CircleGridMap::~CircleGridMap()
{
	if(grid!=NULL){
		for(int i=0; i < m_numOfRows; i++){
			delete [] grid[i];
		}
	}
	delete[] grid;
}

void CircleGridMap::copyFrom(const CircleGridMap& src) throw(bad_alloc){
	m_numOfRows = src.m_numOfRows;
	m_numOfCols = src.m_numOfCols;
	m_xUnits = src.m_xUnits;
	m_yUnits = src.m_yUnits;
	
	int i, j;
	
	grid = new char*[m_numOfRows];
	try{
		for(i=0; i<m_numOfRows; i++){
			grid[i] = new char[m_numOfCols];
		}
	}
	catch(...){
		for(j=0; j<i; j++){
			delete[] grid[j];
		}
		delete[] grid;
		grid=NULL;
		throw bad_alloc();
	}
	
	for(i=0; i<m_numOfRows; i++){
		for(j=0; j<m_numOfCols; j++){
			(*this)(i, j) = src(i, j);
		}
	}
}

char& CircleGridMap::operator()(int i, int j) throw(std::out_of_range){
	if(i< -m_halfRows || i>=m_halfRows){
		std::stringstream expMsg;
		expMsg << "The row number "  << i << " is out of range of the grid";
		throw std::out_of_range(expMsg.str());
	}
	
	if(j< -m_halfCols || j>=m_halfCols ){
		std::stringstream expMsg;
		expMsg << "The column number " << j << " is out of range of the grid";
		throw std::out_of_range(expMsg.str());
	}
    
	int ti = i + m_halfRows;
	int tj = j + m_halfCols;
	//std::cerr << ti << ", " << tj << "\n";
	return grid[ti][tj];
	//return (**(grid+ti*m_numOfRows + tj));
}

const char& CircleGridMap::operator()(int i, int j) const throw(std::out_of_range){
	if(i< -m_halfRows || i>=m_halfRows ){
		std::stringstream expMsg;
		expMsg << "The row number " << i <<" is out of range of the grid";
		throw std::out_of_range(expMsg.str());
	}
	
	if(j< -m_halfCols || j>=m_halfCols ){
		std::stringstream expMsg;
		expMsg << "The column number " << j << " is out of range of the grid";
		throw std::out_of_range(expMsg.str());
	}
    
	int ti = i + m_halfRows;
	int tj = j + m_halfCols;
	return grid[ti][tj];
	//return (**(grid+ti*m_numOfRows + tj));
}

void CircleGridMap::mapOnePoint(float x, float y, int /*cornerCut*/){
	  float fx = x * m_xUnits;
	  float fy = y * m_yUnits;
	  
	  
	  
	  int ix = (int)floor(fx);
	  if (fx > ((float)ix + 0.5f)){
	     ix++;
	  }
	  
	  int iy = (int)floor(fy);
	  if(fy > ((float)iy + 0.5f)){
	    iy++;
	  }
	  //std::cout << ix << "," << iy << std::endl;
	  

	  int i, j;
	  //int xHalfUnits = m_xHalfUnits;
	  //int yHalfUnits = m_yHalfUnits;
	  for(i = ix - m_xUnits-1; i < ix + m_xUnits+1; i++){
	    for(j= iy - m_yUnits-1; j< iy + m_yUnits+1; j++){
	      //if(abs(i-ix)+abs(j-iy) <= m_xUnits + m_yUnits - cornerCut){
	        (*this)(i, j) = '1';
	      //}
	    }
	  }
}

void CircleGridMap::mapListOfPoints(const std::list<DLAElement*>& pList, int /*cornerCut*/){ 
  std::list<DLAElement*>::const_iterator citr;
  float xMin=10000.0f;
  float xMax=-10000.0f;
  float yMin=10000.0f;
  float yMax=-10000.0f;
  float x, y;
  for(citr = pList.begin(); citr!=pList.end(); citr++){
	x = (*citr)->getZ();
	y = (*citr)->getY();
	mapOnePoint(x, y);
	if(x>xMax){
	  xMax = x;
	}
	if(x<xMin){
	  xMin=x;
	}
	if(y>yMax){
		yMax=y;
	}
	if(y<yMin){
		yMin=y;
	}
  }
  
  float fxMin = xMin * m_xUnits;
  float fxMax = xMax * m_xUnits;
  float fyMin = yMin * m_yUnits;
  float fyMax = yMax * m_yUnits;
  
  int ixMin = (int)floor(fxMin);
  if(ixMin-m_xUnits-m_xHalfUnits < -m_halfRows){
	  m_xMin = -m_halfRows; 
  }
  else{
	  m_xMin=ixMin-m_xUnits-m_xHalfUnits;
  }

  int ixMax = (int)floor(fxMax);
  if(ixMax+m_xUnits+m_xHalfUnits > m_halfRows){
	  m_xMax = m_halfRows;
  }
  else{
	  m_xMax=ixMax+m_xUnits+m_xHalfUnits;
  }

  int iyMin = (int)floor(fyMin);
  if(iyMin-m_yUnits-m_yHalfUnits < -m_halfCols){
	  m_yMin = -m_halfCols;
  }
  else{
	  m_yMin=iyMin-m_yUnits-m_yHalfUnits;
  }

  int iyMax = (int)floor(fyMax);
  if(iyMax+m_yUnits+m_yHalfUnits > m_halfCols){
	  m_yMax = m_halfCols;
  }
  else{
	  m_yMax=iyMax+m_yUnits+m_yHalfUnits;
  }
  //std::cout << "Valid area: (" << m_xMin << "," << m_yMin << " : " << m_xMax << "," << m_yMax << ")\n";
}

int CircleGridMap::computeCircumference(){
  int cFerence = 0;
  int i, j, k, t;
  for(i= m_xMin+1; i<m_xMax; i++){
	  for(j= m_yMin+1; j<m_yMax; j++){
		  if((*this)(i,j)=='1'){
			 // std::cout << "i=" << i << ", j=" << j << "\n";
			  for(k=-1; k<=1; k++){
				  for(t=-1; t<=1; t++){
					  if(!(k==0 && t==0)){
						  if(((*this)(i+k, j+t))!='1'){
							  //std::cout << "("<< i+k << ", " << j+t << ") ";
							  ((*this)(i+k, j+t))='2';							  
						  }
					  }
				  }
				  //std::cout << "\n";
			  }
		  }
	  }
  }
  
  for(i=m_xMin; i<=m_xMax; i++){
	  for(j=m_yMin; j<=m_yMax; j++){
		  if((*this)(i,j)=='2'){
			  cFerence++;
		  }
	  }	  
  }
  //cout << cFerence << "\n";
  return cFerence;
}
