#ifndef CIRCLEGRIDMAP_H_
#define CIRCLEGRIDMAP_H_
#include <exception>  //For class exception
#include <stdexcept>  //For standard stl exceptions such as invalid_argument and runtime_exception
#include <new>  //for bad_alloc
#include <list>
#include "Point2D.hpp"

//using std::bad_alloc;
//using std::out_of_range;
//using std::list;

class DLAElement;

class CircleGridMap
{
public:
	CircleGridMap(int numOfRows=1000, int numOfCols=1000, int xUnits=10, int yUnits=10) throw(std::bad_alloc);
	CircleGridMap(const CircleGridMap& src) throw(std::bad_alloc);
	CircleGridMap& operator=(const CircleGridMap& rhs) throw(std::bad_alloc);	
	virtual ~CircleGridMap();
	
	char& operator()(int i, int j) throw(std::out_of_range);
	const char& operator()(int i, int j) const throw(std::out_of_range);
	void reset();	
	int getNumOfRows() const{return m_numOfRows;}
	int getNumOfCols() const{return m_numOfCols;}
	int getXUnits() const{return m_xUnits;}
	int getYUnits() const{return m_yUnits;}
	
	void mapOnePoint(float x, float y, int cornerCut=0);
	void mapListOfPoints(const std::list<DLAElement*>& pList, int cornerCut=0);
	int  computeCircumference();
	
private:
	int m_numOfRows;
	int m_numOfCols;
	int m_xUnits;
	int m_yUnits;
	
	int m_halfRows;
	int m_halfCols;
	int m_xHalfUnits;
	int m_yHalfUnits;
	
	char** grid;

	int m_xMin;
	int m_xMax;
	int m_yMin;
	int m_yMax;
	

private:
	void copyFrom(const CircleGridMap& src) throw(std::bad_alloc);
};

#endif /*CIRCLEGRIDMAP_H_*/
