#include <cmath>
#include <iostream>
#include <cstdlib>
#include "CircularCylinderTemplate.h"
#include "SimuContext.h"
#include "utils.h"

using namespace std;

CircularCylinderTemplate::CircularCylinderTemplate()
  : diagonalLength(0.0f), diagonalAngle(0.0f)
{  
  computeDiagonals();
}

CircularCylinderTemplate::~CircularCylinderTemplate(){}


void CircularCylinderTemplate::computeNewElementCoordinates(){
  SimuContext* context = SimuContext::getInstance();
  float templateDistance = context->getTemplateDistance();
  float aMaxX = aggregate->getMaxX();
  float aMinX = aggregate->getMinX();
  float aMaxY = aggregate->getMaxY();
  float aMinY = aggregate->getMinY();
  float aMaxZ = aggregate->getMaxZ();
  float aMinZ = aggregate->getMinZ();
  float horRadius = fmax(fabs(aMaxX), fabs(aMinX));
  float crossRadius = fmax(fmax(fabs(aMaxY), fabs(aMinY)), fmax(fabs(aMaxZ), fabs(aMinZ)))
                    + templateDistance;
  
 
  float theta = randomAngle();
  //cout << theta << "\n";
  int xRange = (int)(crossRadius+horRadius); // + height*2);
  if(rand()%2==1){  
    x = (float) (rand()%(xRange+1));
  }
  else{
    x = -((float) (rand()%(xRange+1)));  	
  }
  if( x > aMaxX || x < aMinX){
  
    float tx;
    float alpha;
    if(x > 0.0f){
      tx = x - aMaxX; //width/2.0f;
    }
    else{
      tx = fabs(x - aMinX); // width/2.0f;
    }

    while(tx>crossRadius){
      tx -= crossRadius;
    }

    //cout << tx << endl;
    alpha = asin(tx/crossRadius);

    y = crossRadius*cos(alpha) * sin(theta);
    z = crossRadius*cos(alpha) * cos(theta);
  }
  else{
    y = crossRadius*sin(theta);
    z = crossRadius*cos(theta);
  }
}

void CircularCylinderTemplate::computeDiagonals(){
  diagonalLength = sqrt(pow(width/2.0f, 2.0f) + pow(height/2.0f, 2.0f));
  diagonalAngle  = atan(height/width);
}

void CircularCylinderTemplate::expand(float deltaX, float deltaY, float deltaZ){
  DLAReleaseTemplate::expand(deltaX, deltaY, deltaZ);
  computeDiagonals();
}
