#include "CylinderElastin.h"
#include <cmath>
#include <limits>
#include <algorithm>
#include <vector>
#include <iomanip>
#include "utils.h"

CylinderElastin::CylinderElastin(){
  SimuContext* context = SimuContext::getInstance();
  diagonalLength = sqrt( pow(context->getElementWidth(), 2.0f)
                       + pow(context->getElementHeight(), 2.0f)
                       );
}

CylinderElastin::~CylinderElastin(){}

bool CylinderElastin::collideWith(DLAElement* pE){
  float ny = y + vy;
  float nx = x + vx;
  float nz = z + vz;
  return collideTest(nx, ny, nz, pE);
}

bool CylinderElastin::collideTest(float nx, float ny, float nz, DLAElement* pE){
  float xDist = fabs(nx - pE->getX());
  //float yDist = fabs(ny - pE->getY());
  //float zDist = fabs(nz - pE->getZ());
  float yzDist = sqrt((ny - pE->getY())*(ny - pE->getY()) + (nz - pE->getZ())*(nz - pE->getZ()));
  if((xDist <= elementWidth) && (yzDist <= elementHeight)){
     return true;
  }
  return false;
}
/*
void CylinderElastin::stuckWith(DLAElement* pE){
  //float nx = x + vx;
  float xDist = fabs(x - pE->getX());
  float yzDist = sqrt((y - pE->getY())*(y - pE->getY()) + (z - pE->getZ())*(z - pE->getZ()));
  if(yzDist < elementHeight){
    float angle = getYZAngle(pE);
    y = pE->getY()+ elementHeight * sin(angle);
    z = pE->getZ()+ elementHeight * cos(angle);
    if(xDist > elementWidth){
      x = x +vx;
    }
    return;
  }
  else{
    x = x + vx;
    y = y + vy;
    z = z + vz;
    yzDist = sqrt((y - pE->getY())*(y - pE->getY()) + (z - pE->getZ())*(z - pE->getZ()));
    if(yzDist < elementHeight){
      float angle = getYZAngle(pE);
      y = pE->getY()+ elementHeight * sin(angle);
      z = pE->getZ()+ elementHeight * cos(angle);
    }
    return;
  }
}
*/

void CylinderElastin::attach(DLAElement* pE){
	float angle = getYZAngle(pE);
	float newY = y + elementHeight * sin(angle);
	float newZ = z + elementHeight * cos(angle);
	pE->setPos(pE->getX(), newY, newZ);
//	pE->setZ(newZ);
}


bool CylinderElastin::positionOccupied(float angle, float xpos, DLAElement*& pElm){
  list<DLANeighbour>::iterator itr = neighbourList.begin();
  float twopi = GL_PI*2.0f;
  while(itr!= neighbourList.end()){
    if( (fabs((*itr).angle() - angle) < (GL_PI/3.0f) || (twopi - fabs((*itr).angle() - angle)) < (GL_PI/3.0f) )
      && (fabs((*itr).neighbour()->getX() - xpos) < elementWidth) )
      {
      	pElm = (*itr).neighbour();
        return true;
      }
    itr ++;
  }
  pElm = NULL;
  return false;
}

bool CylinderElastin::isAdjustable(DLAElement* pComing, float stepSize, float& adjustAngle, float& adjustDistance, DLAElement*& pOccupy){
  float tAngle = getYZAngle(pComing);
  float initDistance = sqrt(pComing->getY()*pComing->getY() + pComing->getZ()*pComing->getZ());
  float newAngle = tAngle + stepSize;
  float testAngle = fabs(stepSize);
  float minDistance = initDistance;
  float twopi = GL_PI * 2.0f;
  //DLAElement* pOccupy = NULL;
  while( (testAngle < twopi) && (!positionOccupied(newAngle, pComing->getX(), pOccupy))){
  	newAngle = normalAngle(newAngle);
  	float newY = y + elementHeight * sin(newAngle);
  	float newZ = z + elementHeight * cos(newAngle);
  	float newDistance = sqrt(newY*newY + newZ*newZ);
  	if(newDistance < minDistance){
  		minDistance = newDistance;
  		adjustAngle = testAngle;
  	}
  	testAngle += fabs(stepSize);
  	newAngle  = normalAngle(newAngle+stepSize);
  }
  if(fabs(minDistance - initDistance) > EPSILON ){
  	adjustDistance = minDistance;
  	return true;
  }
  //adjustDistance = 0.0f;
  adjustDistance = std::numeric_limits<float>::max();
  return false;
}
