#ifndef DLAELEMENT_H
#define DLAELEMENT_H

#include <iostream>
//#include <fstream>
#include <vector>
#include <list>
#include <map>
#include <string>

class Aggregate;

using namespace std;

class DLAElement;

ostream& operator<<(ostream& ostr, const DLAElement& dlaElement);
ostream& operator<<(ostream& ostr, const DLAElement* dlaElement);

class DLANeighbour{
public:
	
	DLANeighbour():m_angle(-1000.0f),m_neighbour(NULL){		
	}
	
	DLANeighbour(float angle, DLAElement* neighbour)
		:m_angle(angle), m_neighbour(neighbour){		
	}
	
	float angle(){
		return m_angle;
	}
	void setAngle(int angle){
		m_angle = angle;
	}
	
	DLAElement* neighbour(){
		return m_neighbour;
	}

	DLAElement* neighbour() const{
		return m_neighbour;
	}

	
	void setNeighbour(DLAElement* neighbour){
		m_neighbour = neighbour;
	}

private:
  float m_angle;
  DLAElement* m_neighbour;
};

class DLAElement{

  public:
    virtual ~DLAElement();
    DLAElement();

    virtual void  walk(float deltaTime)=0;
    virtual bool  collideWith(DLAElement* dlaElement)=0;
    //virtual void  stuckWith(DLAElement* dlaElement)=0;
    virtual void  attach(DLAElement* dlaElement)=0;
    /**
     * Test whether a new element center x coordinate at xPos can attach at
     * the given angle, if the position is occupied, return the element that
     * occupies this position.
     * 
     */
    virtual bool  positionOccupied(float angle, float xPos, DLAElement*& pElm)=0; 
    virtual bool  isAdjustable(DLAElement* pComing, float stepSize, float& adjustAngle, float& newDistance, DLAElement*& pOccupy)=0;   

    virtual void  stop();
    virtual float centerDistance(DLAElement* dlaElement) const;
    virtual void  save(ostream& outFile);

    float getDiagonalLength();    

    float getX() const;
    void  setX(float value);
    
    float getY() const;
    void  setY(float value);
    
    float getZ() const;
    void  setZ(float value);
    
    float getInitDistanceToCenter();
    float getSpeed() const;
    void  setSpeed(float speed);
    void  setPos(float x=0.0f, float y=0.0f, float z=0.0f);
    void  setInitDistanceToCenter(float dist);
    int   getElementId() const;
    int   getElementId();    
    void  setElementId(int id);
    int   getOrientation() const;
    int   getOrientation();
    void  setOrientation(int orientation);
    float distanceTo(float x=0.0f, float y=0.0f, float z=0.0f);
    bool  isActive() const;
    void  setActive(bool active);
    bool  isValid() const;
    void  setInCollidingArea(bool inArea);
    void addNeighbour(float angle, DLAElement* neighbour);
    //void addNeighbour(float angle, int id);
    list<DLANeighbour>& getNeighbourList();
    int  getNumberOfNeighbours();
    
    //Methods for cross neighbour operations
    void addCrossNeighbour(DLAElement* cNeighbour);
    vector<DLAElement*>& getCrossNeighbours();
    int  getNumberOfCrossNeighbours();
    
    /**
     * Get the center angle with another particle
     * @param target The particle to which the angle will be computed
     * @return The angle in radian representation
     */
    float getYZAngle(DLAElement* target);
    float getYZAngle(float fromY, float fromZ);
    /**
     * Get the center distance in the Y, Z plane
     * @param target The particle to which the distance will be computed
     * @return The distance
     */
    float getYZDistance(DLAElement* target);
    float overlappedLength(DLAElement* target);
    float overlappedLength(float xPos);    
    
    bool  isNeighbour(DLAElement* target);
    bool  inNeighbourList(DLAElement* target);

    //used by findElementById in the aggregate
    //bool  equalId(int id);
    DLAElement* findNeighbourById(int id);
    bool  removeNeighbourById(int id);
    //Detach an element from all its neighbours, and clear the element's neighbour list.
    void  detachFromNeighbours();
    //bool collideWithTargetNeighbours(float newx, float newy, float newz, DLAElement* target);

  protected:
    static int idSerialNumber;
    float x;
    float y;
    float z;
    float vx;
    float vy;
    float vz;
    float elementWidth;
    float elementHeight;
    float elementDepth;
    float initDistanceToCenter;
    float speed;
    float diagonalLength;
    bool  active;
    bool  valid; 
    bool  inCollidingArea;
    int   elementId;
    int   orientation; // 0: The hydrophobic and cross link motif is the same as 
                       //    the order that is specified in the configuration file (from left to right)
                       // 1: Reverse to the configuration
    //map<string, vector<DLAElement*> > neighbours;    
    list<DLANeighbour> neighbourList;
    //Neighbours having cross-linking contact with this element.
    vector<DLAElement*> crossNeighbours;
};

class DLA3DElement: public DLAElement{

  public:
    virtual ~DLA3DElement();
    DLA3DElement();
    
    virtual void walk(float deltaTime);
};
#endif
