#include "DLAReleaseTemplate.h"
#include "DLAElement.h"
#include "CuboidElastin.h"
#include "CylinderElastin.h"
#include "SimuContext.h"
#include <iostream>
#include <cmath>
#include <cstdlib>

using namespace std;

DLAReleaseTemplate::DLAReleaseTemplate()
  :x(0.0), y(0.0), z(0.0)
{
  SimuContext* context = SimuContext::getInstance();
  width  = context->getTemplateWidth();
  height = context->getTemplateHeight();
  depth  = context->getTemplateDepth();
}

DLAReleaseTemplate::~DLAReleaseTemplate(){}

DLAElement* DLAReleaseTemplate::generateElement(string elementType){
  computeNewElementCoordinates();
  DLAElement* p=NULL;

  if(recycleList.size()>0){
    vector<DLAElement*>::iterator itr;
    itr = recycleList.begin();
    p = *itr;
    recycleList.erase( itr );
    p->setActive(true);
    p->setPos(x, y, z);
    p->setInitDistanceToCenter(p->distanceTo(0.0f, 0.0f, 0.0f));    
  }
  else{
    if(elementType == "CuboidElastin"){
      p = new CuboidElastin();
      p->setPos(x, y, z);
      p->setInitDistanceToCenter(p->distanceTo(0.0f, 0.0f, 0.0f));    
      //return p;
    }
    else if(elementType == "CylinderElastin"){
      p = new CylinderElastin();
      p->setPos(x, y, z);
      p->setInitDistanceToCenter(p->distanceTo(0.0f, 0.0f, 0.0f)); 
      SimuContext* context = SimuContext::getInstance();
      if(context->getElementOrientation()==1){
        if(rand()%2==1){
          p->setOrientation(1);
        }   
        else{
          p->setOrientation(0);
        }
      }
      //return p;
    }
  }
  return p;
}

void DLAReleaseTemplate::addToRecycleList(DLAElement* recyclableElement){
  recycleList.push_back(recyclableElement);
}

void DLAReleaseTemplate::setAggregate(Aggregate* agg){
  aggregate = agg;
}
void DLAReleaseTemplate::expand(float deltaX, float deltaY, float deltaZ){
  width  += deltaX;
  height += deltaY;
  depth  += deltaZ;
//  cout << "Template: " << width << ", " << height << ", " << depth << endl;
}

void DLAReleaseTemplate::expand(int axes){ // 1: x-axes, 2: y-axes, 3: z-axes
  SimuContext* context = SimuContext::getInstance();
  if(axes==1){
    width += context->getTemplateDistance();
    height = width * context->getElementHeight()/context->getElementWidth();
    depth  =  width * context->getElementDepth()/context->getElementWidth();
  } 
  if(axes==2){
    height += context->getTemplateDistance();
    width  = height * context->getElementWidth()/context->getElementHeight();
    depth  = height * context->getElementDepth()/context->getElementHeight();
  } 
  if(axes==3){
    depth += context->getTemplateDistance();
    width  = depth * context->getElementWidth()/context->getElementDepth();
    height = depth * context->getElementHeight()/context->getElementDepth();
  } 
  cout << "Template: " << width << ", " << height << ", " << depth << endl;
}

float DLAReleaseTemplate::getWidth(){
  return width;
}

float DLAReleaseTemplate::getHeight(){
  return height;
}

float DLAReleaseTemplate::getDepth(){
  return depth;
}

void DLAReleaseTemplate::computeNewElementCoordinates(){}

