#ifndef DLASERIALIZATION_H
#define DLASERIALIZATION_H
/* This is the header file including all the functions used for object serialization
   during real-time visualization.
*/
#include "SimuContext.h"
#include "Aggregate.h"
#include "CylinderElastin.h"
#include <QDataStream>
#include <QDebug>

inline QDataStream& operator<<(QDataStream& qout, const ElementTemplate& elmTpl){
    qout << (quint8)elmTpl.exonType() << (qreal)elmTpl.exonLength() << (qreal)elmTpl.simuLength();
    return qout;
}

inline QDataStream& operator>>(QDataStream& qin, ElementTemplate& elmTpl){
    quint8 tmpChar;
    qin >> tmpChar;
    elmTpl.setExonType((char)tmpChar);
    
    qreal tmpReal;
    qin >> tmpReal;
    elmTpl.setExonLength((float)tmpReal);
    
    qin >> tmpReal;
    elmTpl.setSimuLength((float)tmpReal);
    
    return qin;
}


inline QDataStream& operator<<(QDataStream& qout, const SimuContext& context){

    qout << (qint32)context.getNumberOfElements()    << (qreal)context.getDaysOfSimuTime();
    qDebug() << (qint32)context.getNumberOfElements();
    qDebug() << (qreal)context.getDaysOfSimuTime();
    qout << (qreal)context.getTemplateDistance()     << (qreal)context.getTemplateExpandStep();
    qout << (qreal)context.getTemplateAdjustRatio()  << (qreal)context.getTemplateWidth();
    qout << (qreal)context.getTemplateHeight()       << (qreal)context.getTemplateDepth();
    
    qout << (quint32)context.getElementOrientation() << (qreal)context.getElementWidth();
    qout << (qreal)context.getElementHeight()        << (qreal)context.getElementDepth();
    qout << (qreal)context.getElementSpeed();
    
    qout << (quint32)context.getCenterAdjustment();
    
    qout << (quint32)context.getDiffusionFlag()       << (quint32)context.getDiffusionLevel();
    qout << (qreal)context.getDiffusionStep()         << (qreal)context.getDiffusionKTConstant();
    qout << (quint32)context.getDiffusionTryTimes()   << (qreal)context.getDiffusionDistanceThreshold();
    qout << (qreal)context.getDiffusionInfluenceCoefficient();
    
    qout << (quint32)context.getHydrophobicFlag()          << (quint32)context.getHydroConstraintType();
    qout << (qreal)context.getHydroConstraintCoefficient() << (qreal)context.getHydroConstraintConstant();
    
    const vector<ElementTemplate>* tempVector = context.getElementTemplate();
    vector<ElementTemplate>::const_iterator itr;
    for(itr=tempVector->begin(); itr!=tempVector->end(); itr++){
        qout << (*itr);
    }
    return qout;
}

inline QDataStream& operator>>(QDataStream& qin, SimuContext& context){
    qint32 tmpInt;
    qreal  tmpReal;
    
    qin >> tmpInt;   context.setNumberOfElements((int)tmpInt);
    qDebug() << tmpInt;
    qin >> tmpReal;  context.setDaysOfSimuTime((float)tmpReal);
    qDebug() << tmpReal;
    qin >> tmpReal;  context.setTemplateDistance((float)tmpReal);
    qin >> tmpReal;  context.setTemplateExpandStep((float)tmpReal);
    qin >> tmpReal;  context.setTemplateAdjustRatio((float)tmpReal);
    qin >> tmpReal;  context.setTemplateWidth((float)tmpReal);
    qin >> tmpReal;  context.setTemplateHeight((float)tmpReal);
    qin >> tmpReal;  context.setTemplateDepth((float)tmpReal);
    
    qin >> tmpInt;   context.setElementOrientation((int)tmpInt);
    qin >> tmpReal;  context.setElementWidth((int)tmpReal);
    qin >> tmpReal;  context.setElementHeight((float)tmpReal);
    qin >> tmpReal;  context.setElementDepth((float)tmpReal);
    qin >> tmpReal;  context.setElementSpeed((float)tmpReal);
    qin >> tmpInt;   context.setCenterAdjustment((int)tmpInt);
    
    qin >> tmpInt;   context.setDiffusionFlag((int)tmpInt);
    qin >> tmpInt;   context.setDiffusionLevel((int)tmpInt);
    qin >> tmpReal;  context.setDiffusionStep((float)tmpReal);
    qin >> tmpReal;  context.setDiffusionKTConstant((float)tmpReal);
    qin >> tmpInt;   context.setDiffusionTryTimes((int)tmpInt);
    qin >> tmpReal;  context.setDiffusionDistanceThreshold((float)tmpReal);
    qin >> tmpReal;  context.setDiffusionInfluenceCoefficient((float)tmpReal);
    qin >> tmpInt;   context.setHydrophobicFlag((int)tmpInt);
    qin >> tmpInt;   context.setHydroConstraintType((int)tmpInt);
    qin >> tmpReal;  context.setHydroConstraintCoefficient((float)tmpReal);
    qin >> tmpReal;  context.setHydroConstraintConstant((float)tmpReal);
    
    vector<ElementTemplate> tplVector;
    ElementTemplate tmpTpl;
    while(!qin.atEnd()){
        qin >> tmpTpl;
        tplVector.push_back(tmpTpl);
    }
    context.setElementTemplate(tplVector);
    return qin;
}

inline QDataStream& operator<<(QDataStream& qout, const DLAElement& dlaElm){
    qout << (qint32)dlaElm.getElementId() << (qreal)dlaElm.getX() << (qreal)dlaElm.getY() << (qreal)dlaElm.getZ() << (qreal)dlaElm.getSpeed();
    return qout;
}

inline QDataStream& operator>>(QDataStream& qin, DLAElement& dlaElm){
    qint32 tmpInt;
    qreal  tmpReal;
    
    qin >> tmpInt;  dlaElm.setElementId((int)tmpInt);
    
    qin >> tmpReal; dlaElm.setX((float)tmpReal);
    qin >> tmpReal; dlaElm.setY((float)tmpReal);
    qin >> tmpReal; dlaElm.setZ((float)tmpReal);
    qin >> tmpReal; dlaElm.setSpeed((float)tmpReal);
    return qin;
}

inline QDataStream& operator<<(QDataStream& qout, const Aggregate& agg){
    const list<DLAElement*>* dlaList = agg.getElements();
    list<DLAElement*>::const_iterator itr;
    
    for(itr=dlaList->begin(); itr!=dlaList->end(); itr++){
    	//std::cout << (*(*itr));
        qout << (*(*itr));
    }
    return qout;
}

inline QDataStream& operator>>(QDataStream& qin, Aggregate& agg){
    while(!qin.atEnd()){
        DLAElement* pElm = new CylinderElastin();
        qin >> (*pElm);
        agg.addElement(pElm);
    }
    return qin;
}
#endif
