/*
 * Template class for 3D vector
 * 
 * Bill (Hongyan) Song
 * 
 * Feb, 2008
 * */
 
#ifndef DVECTOR_H_
#define DVECTOR_H_

#include <iostream>
using std::ostream;
using std::istream;

// forward delcaration for friend template functions

template <typename T>
class DLAVector;

//Addition
template <typename T>
const DLAVector<T> operator+(const DLAVector<T>& lhs, const DLAVector<T>& rhs);

//Substraction
template <typename T>
const DLAVector<T> operator-(const DLAVector<T>& lhs, const DLAVector<T>& rhs);

//Cross product
template <typename T>
const DLAVector<T> operator*(const DLAVector<T>& lhs, const DLAVector<T>& rhs);

//Dot product
template <typename T>
T operator^(const DLAVector<T>& lhs, const DLAVector<T>& rhs);

//Equal
template <typename T>
bool operator==(const DLAVector<T>& lhs, const DLAVector<T>& rhs);

//Insertion
template <typename T>
ostream& operator<<(ostream& ostr, const DLAVector<T>& vec);

//Extraction
template <typename T>
istream& operator>>(istream& istr, DLAVector<T>& vec);

template <typename T>
class DLAVector
{
public:
	DLAVector();
	DLAVector(T x, T y, T z);
	virtual ~DLAVector();
	friend const DLAVector<T> operator+ <T>(const DLAVector<T>& lhs, const DLAVector<T>& rhs);
	friend const DLAVector<T> operator- <T>(const DLAVector<T>& lhs, const DLAVector<T>& rhs);	
	friend const DLAVector<T> operator* <T>(const DLAVector<T>& lhs, const DLAVector<T>& rhs);	
  friend T operator^ <T>(const DLAVector<T>& lhs, const DLAVector<T>& rhs);	
	friend bool operator== <T>(const DLAVector<T>& lhs, const DLAVector<T>& rhs);
	friend ostream& operator<< <T>(ostream& ostr, const DLAVector<T>& vec);
	friend istream& operator>> <T>(istream& istr, DLAVector<T>& vec);
	
//	template <typename U>
//	U length();
	
	T getX();
	void setX(T x);
	
	T getY();
	void setY(T y);
	
	T getZ();
	void setZ(T z);	
	
protected:
  T x;
  T y;
  T z;
};

template <typename T>
DLAVector<T>::DLAVector()
{
}

template <typename T>
DLAVector<T>::DLAVector(T x, T y, T z){
	setX(x);
	setY(y);
	setZ(z);
}

template <typename T>
DLAVector<T>::~DLAVector()
{	
}

//template <typename T>
//float DLAVector<T>::length(){
//	return sqrt(static_cast<float>(x*x + y*y + z*z));
//}

template <typename T>
T DLAVector<T>::getX(){
	//T& tx = x;
	return x;
}

template <typename T>
void DLAVector<T>::setX(T x){
	this->x = x;
}

template <typename T>
T DLAVector<T>::getY(){
	//T& ty = y;
	return y;
}

template <typename T>
void DLAVector<T>::setY(T y){
	this->y = y;
}

template <typename T>
T DLAVector<T>::getZ(){
	//T& tz = z;
	return z;
}

template <typename T>
void DLAVector<T>::setZ(T z){
	this->z = z;
}

template <typename T>
const DLAVector<T> operator+(const DLAVector<T>& lhs, const DLAVector<T>& rhs){
	DLAVector<T> newVector;
	newVector.x = lhs.x + rhs.x;
	newVector.y = lhs.y + rhs.y;
	newVector.z = lhs.z + rhs.z;
	return newVector;
}

template <typename T>
const DLAVector<T> operator-(const DLAVector<T>& lhs, const DLAVector<T>& rhs){
	DLAVector<T> newVector;
	newVector.x = lhs.x - rhs.x;
	newVector.y = lhs.y - rhs.y;
	newVector.z = lhs.z - rhs.z;
	return newVector;
}

template <typename T>
const DLAVector<T> operator*(const DLAVector<T>& lhs, const DLAVector<T>& rhs){
	DLAVector<T> newVector;
	newVector.x = lhs.y * rhs.z - lhs.z * rhs.y;
	newVector.y = -(lhs.x * rhs.z -lhs.z * rhs.x) ;
	newVector.z = lhs.x * rhs.y - lhs.y * rhs.x;
	return newVector;
}


template <typename T>
T operator^(const DLAVector<T>& lhs, const DLAVector<T>& rhs){
	return (lhs.x * rhs.x + lhs.y * rhs.y + lhs.z * rhs.z);
}

template <typename T>
bool operator==(const DLAVector<T>& lhs, const DLAVector<T>& rhs){
	return (  (lhs.x == rhs.x) 
	       && (lhs.y == rhs.y)
	       && (lhs.z == rhs.z)
	       );
}

template <typename T>
ostream& operator<<(ostream& ostr, const DLAVector<T>& vec){
	ostr << "(" << vec.x << "," << vec.y << "," << vec.z << ")";
	return ostr; 
}

template <typename T>
istream& operator>>(istream& istr, DLAVector<T>& vec){	
	istr >> vec.x >> vec.y >> vec.z;
	return istr;
}


#endif /*DVECTOR_H_*/
