#include "DataCollector.h"
#include <cmath>

ostream& operator<<(ostream& outStr, const DataCollector& dCollector){
	outStr << "Statistical Data: \n";
	outStr << "---------------------------------------------\n";
	outStr << "  Average Time: " << dCollector.averageTime << "\n";
	outStr << "  Density: "  << dCollector.density << "\n";
	outStr << "  Surface Area: " << dCollector.surfaceArea << "\n";
	outStr << "  Average Connectivity: " << dCollector.averageConnectivity << "\n";
    outStr << "  Aggregate Length: " << dCollector.aggLength << "\n"; 
    outStr << "  Aggregate Cross Neighbours: " << dCollector.averageCrossNeighbours << "\n";
    
	outStr << "  Connectivity:\n";
	map<int, int>::const_iterator itr = dCollector.connectivityMap.begin();
	while(itr!=dCollector.connectivityMap.end()){
		outStr << "\t" << (*itr).first << "\t\t" << (*itr).second << "\n";
		itr++;
	}
	
	outStr << "  Distance to center:\n";
	map<float, int>::const_iterator fitr = dCollector.distToCenterMap.begin();
	while(fitr!=dCollector.distToCenterMap.end()){
		outStr << "\t" << (*fitr).first << "\t\t" << (*fitr).second << "\n";		
		fitr++;
	}
	return outStr;
}

DataCollector::DataCollector()
  :pAggregate(NULL),averageTime(0.0f), density(0.0f), surfaceArea(0.0f){
}

DataCollector::DataCollector(Aggregate* pAgg)
  :pAggregate(pAgg), averageTime(0.0f), density(0.0f), surfaceArea(0.0f){
}

DataCollector::~DataCollector(){
}

Aggregate* DataCollector::getAggregate(){
	return pAggregate;
}

void DataCollector::setAggregate(Aggregate* pAgg){
	pAggregate = pAgg;
}

void DataCollector::collectData(){
	if(pAggregate==NULL){
		cerr << "No data to collect, please setAggregate() first !" << endl;
		return;
	}
  averageTime = float(pAggregate->getSecondsSpent())/float(pAggregate->getNumberOfElements());
  density = pAggregate->computeAverageDensity(10.0f);
  surfaceArea = pAggregate->computeSurfaceArea(10.0f);
  averageConnectivity = pAggregate->computeConnectivity();
  aggLength = fabs(pAggregate->getMaxX() - pAggregate->getMinX());
  
  averageCrossNeighbours = pAggregate->computeCrossLinks(1); //computeNumberOfOverlappedDomainPartsWithNeighbours('c');
  //averageCrossNeighbours = pAggregate->computeCrossNeighbours(0.1f);
  pAggregate->collectConnectivityInfo(connectivityMap);
  pAggregate->collectDistToCenterInfo(distToCenterMap);
}
/*
ostream& DataCollector::save(ostream& outStr){
	outStr << "<statistical-data> \n";
	outStr << "  <density>"  << density << "</density>\n";
	outStr << "  <surface-area>" << surfaceArea << "</surface-area>\n";
	outStr << "  <average-connectivity>" << averageConnectivity << "</average-connectivity>\n"; 
	outStr << "  <length>" << aggLength << "</length>\n";	
	outStr << "  <connectivity>\n";
	map<int, int>::const_iterator itr = connectivityMap.begin();
	while(itr!=connectivityMap.end()){
		outStr << "    <connect-item number-of-neighbours=\"" << (*itr).first << "\" number-of-particles=\"" << (*itr).second << "\"/>\n";
		itr++;
	}
	outStr << "  </connectivity>\n";
	
	outStr << "  <distance-to-center> \n";
	map<float, int>::const_iterator fitr = distToCenterMap.begin();
	while(fitr!=distToCenterMap.end()){
		outStr << "    <distance-item distance=\"" << (*fitr).first << "\" number-of-particles=\"" << (*fitr).second << "\"/>\n";		
		fitr++;
	}
	outStr << "  </distance-to-center>\n";
		
	outStr << "</statistical-data>\n";	
	return outStr;
}
*/
