#ifndef SIMUCONTEXT_H
#define SIMUCONTEXT_H

//#include <xercesc/dom/DOM.hpp>
//#include <xercesc/parsers/XercesDOMParser.hpp>
#include <string>
#include <vector>
#include <list>
#include <iostream>
//#include <fstream>
#include <stdexcept>
#include <cmath>

#include "utils.h"

using namespace std;
//using namespace xercesc;
/*
typedef struct{
  char   exonType;  
  float  exonLength;
  float  simuLength;  
} ElementTemplate;
*/

class ElementTemplate{
	
public:
  char  exonType() const {return m_exonType;}
  float exonLength() const {return m_exonLength;}
  float simuLength() const {return m_simuLength;}
  float centerDistance() const {return m_centerDistance;}
  float reverseCenterDistance() const {return m_reverseCenterDistance;}
  
  void  setExonType(char value){ m_exonType = value; }
  void  setExonLength(float value){ m_exonLength = value; }
  void  setSimuLength(float value){ m_simuLength = value; }
  void  setCenterDistance(float value){ m_centerDistance = value;}
  void  setReverseCenterDistance(float value){ m_reverseCenterDistance = value;}
  
  
private:
  char   m_exonType;  
  float  m_exonLength;
  float  m_simuLength;
  float  m_centerDistance;
  float  m_reverseCenterDistance;
};

/*
typedef struct{
  float  lower;  
  float  upper;
  float  probability;
} HydroConstraint;
*/

class SimuContext
{
  public:
    virtual ~SimuContext();
    
    int   getNumberOfElements() const;
    void  setNumberOfElements(int value);
    float getDaysOfSimuTime() const {return daysOfSimuTime;}
    void  setDaysOfSimuTime(float value) { daysOfSimuTime=value;}
    float getTemplateDistance() const;
    void  setTemplateDistance(float distance);
    float getTemplateExpandStep() const;
    void  setTemplateExpandStep(float step);
    float getTemplateAdjustRatio() const;
    void  setTemplateAdjustRatio(float ratio);
    float getTemplateWidth() const;
    void  setTemplateWidth(float width);
    float getTemplateHeight() const;
    void  setTemplateHeight(float height);
    float getTemplateDepth() const;
    void  setTemplateDepth(float depth);
    int   getElementOrientation() const;
    void  setElementOrientation(int orientation);
    float getElementWidth() const;
    void  setElementWidth(float width);
    float getElementHeight() const;
    void  setElementHeight(float height);
    float getElementDepth() const;
    void  setElementDepth(float depth);
    float getElementSpeed() const;
    void  setElementSpeed(float speed);

    int   getCenterAdjustment() const;
    void  setCenterAdjustment(int adjustment);
    int   getHydrophobicFlag() const;
    void  setHydrophobicFlag(int hydrophobiFlag);
    int   getHydroConstraintType() const {return hydroConstraintType;}
    void  setHydroConstraintType(int conType) {hydroConstraintType=conType;}
    float getHydroConstraintCoefficient() const {return hydroConstraintCoefficient;}
    void  setHydroConstraintCoefficient(float coefficient){hydroConstraintCoefficient=coefficient;}
    float getHydroConstraintConstant() const {return hydroConstraintConstant;}
    void  setHydroConstraintConstant(float constant){hydroConstraintConstant=constant;}
    float getLinearHydroProbability(float hydroPercentage) const{//type 1
	        return hydroConstraintCoefficient * hydroPercentage + hydroConstraintConstant;
    }
    float getExponentialDensityHydroProbability(float hydroPercentage) const { //type 2
	        return hydroConstraintCoefficient*exp(-hydroConstraintCoefficient*hydroPercentage);
    }
    float getExponentialCumulativeHydroProbability(float hydroPercentage) const { //type 3 
	        return (1 - exp(-hydroConstraintCoefficient*hydroPercentage));
    }
    float getPowerHydroProbability(float hydroPercentage) const{ //type 4
    		return pow(hydroPercentage, hydroConstraintCoefficient);
    }
    float getNormalHydroProbability(float hydroPercentage) const{ //type 5, normal distribution
    	return (1.0f/sqrt(hydroConstraintCoefficient*2.0f*GL_PI))*exp(-pow((hydroPercentage-0.5f), 2.0f)/(2.0f*hydroConstraintCoefficient)) - hydroConstraintConstant;
    }
    //vector<HydroConstraint>* getHydroConstraints();
    //float getHydroProbability(float hydroPercentage);
    int   getDiffusionFlag() const;
    void  setDiffusionFlag(int diffusionFlag);
    int   getDiffusionLevel() const;
    void  setDiffusionLevel(int level);
    float getDiffusionStep() const; 
    void  setDiffusionStep(float diffusionStep);
    float getDiffusionKTConstant() const;
    void  setDiffusionKTConstant(float ktConstant);
    int   getDiffusionTryTimes() const {return diffusionTryTimes;}
    void  setDiffusionTryTimes(int times){diffusionTryTimes=times;}
    float getDiffusionDistanceThreshold() const {return diffusionDistanceThreshold;}
    void  setDiffusionDistanceThreshold(float threshold){ diffusionDistanceThreshold=threshold;}
    float getDiffusionInfluenceCoefficient() const {return diffusionInfluenceCoefficient;}
    void  setDiffusionInfluenceCoefficient(float coeff){ diffusionInfluenceCoefficient=coeff;}
    
    float getSimuHydrophobicLength() const;
    //float getSimuCrosslinkLength() const;
    float getAverageCrossDomainLength() const;
    float getLeftPart(float length, list<ElementTemplate>& lConnectPart) const;
    float getReverseLeftPart(float length, list<ElementTemplate>& lConnectPart) const;
    float getRightPart(float length, list<ElementTemplate>& rConnectPart) const;
    float getReverseRightPart(float length, list<ElementTemplate>& rConnectPart) const;
    void  renderDomainPart(list<ElementTemplate>& domainPart) const;
    void  renderTemplateDomainPart(vector<ElementTemplate>& domainPart) const;  

    const vector<ElementTemplate >* getElementTemplate() const;
    vector<ElementTemplate >* getElementTemplate();
    void setElementTemplate(const vector<ElementTemplate>& elmTemplate){elementTemplate=elmTemplate;}
    void save(ostream& outFile) const;
    static SimuContext* getInstance();
    static void	 destroyCurrentContext();
    void     computeSimuLength();
    //Compute the distance between domain part centers to the template center
    // cccccchhhhhhcccccccchhhhhhhcccc
    //    6     6      8      7     4
    //Template center: (6+6+8+7+4)/2=15.5
    //Orientation=0: then distances are
    //    6/2-15.5=-12.5
    //    6+6/2-15.5=-3.5
    //    6+6+8/2-15.5=1.5
    //    6+6+8+7/2-15.5=8
    //    6+6+8+7+4/2-15.5=13.5
    //If orientation=1, the computation begains in the reverse order from 4, 7, 8, 6, 6
    void computeTemplateCenterDistance();    

  protected:
    string contextFile;
    //XercesDOMParser* contextParser;

    int   numberOfElements;
    float daysOfSimuTime;
    float templateDistance;
    float templateExpandStep;
    float templateAdjustRatio;
    float templateWidth;
    float templateHeight;
    float templateDepth;

    int   elementOrientation;
    float elementWidth;
    float elementHeight;
    float elementDepth;
    float elementSpeed;

    int   centerAdjustment;
    float simuHydrophobicLength;
    float averageCrossDomainLength;
    
    //Parameters for surface diffusion
    int   diffusionFlag;
    int   diffusionLevel;
    float diffusionStep;
    float diffusionKTConstant;
    int   diffusionTryTimes;
    float diffusionDistanceThreshold;
    float diffusionInfluenceCoefficient;

    //Parameters for hydrophobic constraints
    int   hydrophobicFlag;
    int   hydroConstraintType;
    float hydroConstraintCoefficient;
    float hydroConstraintConstant;
        
    vector<ElementTemplate> elementTemplate;
    static   SimuContext* simuInstance;
    
    // The empty head for the element template list
    ElementTemplate tmpHead;
    
    // Protected constructor for preventing creation of multiple instances
    // Singleton Pattern
    SimuContext();
};
#endif
