#include <iostream>
#include <math.h>
#include <time.h>
#include "utils.h"
//#include "CylinderTemplate.h"
#include "CircularCylinderTemplate.h"
#include "SimuModel.h"
#include "DataCollector.h"
#include "XMLFileManager.h"

using namespace std;
 
SimuModel::SimuModel()
: profileName(""), traceFileName(""), context(NULL), releaseTemplate(NULL), aggregate(NULL), activeElement(NULL)
{
  //srand ( time(NULL) );
}

// Constructor for models built from a original profile
SimuModel::SimuModel(string profileName, string traceFileName)
  : context(NULL), aggregate(NULL), activeElement(NULL)
{
  this->profileName = profileName;
  if(traceFileName.empty()){
	  traceFileName = "trace.xml";
  }
  this->traceFileName = traceFileName;
  
  context = SimuContext::getInstance();
  XMLFileManager xManager;
  xManager.loadContext(profileName, context);
  
  elementWidth  = context->getElementWidth();
  halfElementWidth = elementWidth/2.0f;
  elementHeight = context->getElementHeight();
  halfElementHeight = elementHeight/2.0f;
  elementDepth  = context->getElementDepth();
  halfElementDepth = elementDepth/2.0f;
  
  templateDistance = context->getTemplateDistance();
  twoTimesDistance = templateDistance * 2.0f;
  aggregate = new Aggregate();
  releaseTemplate = new CircularCylinderTemplate();
  releaseTemplate->setAggregate(aggregate);
  // releaseTemplate = new CylinderTemplate();
  if(releaseTemplate==NULL){
    cout << "Generating Release Template error ...... " << endl;
    //return false;
  }

  DLAElement* p = releaseTemplate->generateElement("CylinderElastin");
  p->setPos(0.0f, 0.0f, 0.0f);
  aggregate->addElement(p); 
  maxX = aggregate->getMaxX() + halfElementWidth;
  minX = aggregate->getMinX() - halfElementWidth;
  maxY = aggregate->getMaxY() + halfElementHeight;
  minY = aggregate->getMinY() - halfElementHeight;
  maxZ = aggregate->getMaxZ() + halfElementDepth;
  minZ = aggregate->getMinY() - halfElementDepth;  
}

// Constructor for models built from a previous generated trace file
SimuModel::SimuModel(string traceFileName)
	: context(NULL), aggregate(NULL), activeElement(NULL)
{
	this->traceFileName = traceFileName;
	
	context = SimuContext::getInstance();
	aggregate = new Aggregate();
	XMLFileManager xmlManager;
	xmlManager.loadTrace(traceFileName, context, aggregate);
	//context->save(cout);
	//aggregate->save(cout);
	elementWidth  = context->getElementWidth();
	halfElementWidth = elementWidth/2.0f;
	elementHeight = context->getElementHeight();
	halfElementHeight = elementHeight/2.0f;
	elementDepth  = context->getElementDepth();
	halfElementDepth = elementDepth/2.0f;
	  
	templateDistance = context->getTemplateDistance();
	twoTimesDistance = templateDistance * 2.0f;

	releaseTemplate = new CircularCylinderTemplate();
	releaseTemplate->setAggregate(aggregate);
	// releaseTemplate = new CylinderTemplate();
	if(releaseTemplate==NULL){
	  cout << "Generating Release Template error ...... " << endl;
	  //return false;
	}

	//  DLAElement* p = releaseTemplate->generateElement("CylinderElastin");
	//  p->setPos(0.0f, 0.0f, 0.0f);
	//  aggregate->addElement(p); 
	maxX = aggregate->getMaxX() + halfElementWidth;
	minX = aggregate->getMinX() - halfElementWidth;
	maxY = aggregate->getMaxY() + halfElementHeight;
	minY = aggregate->getMinY() - halfElementHeight;
	maxZ = aggregate->getMaxZ() + halfElementDepth;
	minZ = aggregate->getMinY() - halfElementDepth;	
}

SimuModel::~SimuModel(){
  if(aggregate!=NULL) delete aggregate;
  if(releaseTemplate!=NULL) delete releaseTemplate;
}
/*
bool SimuModel::setup(){
  context = SimuContext::getInstance();
  XMLFileManager xManager;
  xManager.loadContext(profileName, context);
	
  //context->loadContext(profileName);
  elementWidth  = context->getElementWidth();
  halfElementWidth = elementWidth/2.0f;
  elementHeight = context->getElementHeight();
  halfElementHeight = elementHeight/2.0f;
  elementDepth  = context->getElementDepth();
  halfElementDepth = elementDepth/2.0f;
  
  templateDistance = context->getTemplateDistance();
  twoTimesDistance = templateDistance * 2.0f;
  aggregate = new Aggregate();
  releaseTemplate = new CircularCylinderTemplate();
  releaseTemplate->setAggregate(aggregate);
  // releaseTemplate = new CylinderTemplate();
  if(releaseTemplate==NULL){
    cout << "Generating Release Template error ...... " << endl;
    return false;
  }

  DLAElement* p = releaseTemplate->generateElement("CylinderElastin");
  p->setPos(0.0f, 0.0f, 0.0f);
  aggregate->addElement(p); 
  maxX = aggregate->getMaxX() + halfElementWidth;
  minX = aggregate->getMinX() - halfElementWidth;
  maxY = aggregate->getMaxY() + halfElementHeight;
  minY = aggregate->getMinY() - halfElementHeight;
  maxZ = aggregate->getMaxZ() + halfElementDepth;
  minZ = aggregate->getMinY() - halfElementDepth;

  return true;  
}
*/

// Modified version
bool SimuModel::isOutOfBoundary(DLAElement* p){
  //float initDistance = p->getgetInitDistanceToCenter()();
  float curDistance = p->distanceTo(0.0f, 0.0f, 0.0f); 
  if((curDistance - p->getInitDistanceToCenter()) > twoTimesDistance){
    return true;
  }
  return false;
}

bool SimuModel::inCollidingArea(DLAElement* p){
  float speed = p->getSpeed() * 2.0f;
  if(  (p->getX() <= maxX + speed)
    && (p->getX() >= minX - speed)
    && (p->getY() <= maxY + speed)
    && (p->getY() >= minY - speed)
    && (p->getZ() <= maxZ + speed)
    && (p->getZ() >= minZ - speed)
    ){
      return true;
  }
  return false;
}

bool SimuModel::templateIsToExpand(){
  bool expanded = false;
  if( (templateDistance-releaseTemplate->getWidth()/2 > aggregate->getMinX())
    || (releaseTemplate->getWidth()/2.0f - templateDistance < aggregate->getMaxX())
    ){
      releaseTemplate->expand(context->getTemplateExpandStep(), 0.0f, 0.0f);
      expanded = true;    
    }
    
  if( (templateDistance - releaseTemplate->getHeight()/2 > aggregate->getMinY())
    || (releaseTemplate->getHeight()/2 - templateDistance < aggregate->getMaxY())
    ){
      releaseTemplate->expand(0.0f, context->getTemplateExpandStep(), 0.0f);
      expanded = true;
    }

  if( (templateDistance - releaseTemplate->getDepth()/2 > aggregate->getMinZ())
    || (releaseTemplate->getDepth()/2 - templateDistance < aggregate->getMaxZ())
    ){
      releaseTemplate->expand(0.0f, 0.0f, context->getTemplateExpandStep());
      expanded = true;
    }

  if(expanded){
  	return true;
  }
  return false;
}

void SimuModel::run(){
  time_t beginTime, endTime, curTime;
  struct tm *timeInfo;
  int remains=0;
  int elapsedSeconds=0;
  //localtime(&curTime);
  //cout << "Simulation begins at "  << asctime(timeInfo) << endl;
  map<float, list<DLAElement*> >* pMap = aggregate->getElementMap();
  if(pMap->find(1.0f)==pMap->end()){
    list<DLAElement* > pList;
    (*pMap)[1.0f] = pList;
  }
  //emit templateChanged(releaseTemplate->getWidth(), releaseTemplate->getHeight());
  int currentNumberOfElements = aggregate->getNumberOfElements();
  int secondsOfSimuTime      = (int)(context->getDaysOfSimuTime() * 24.0f * 60.0f * 60.0f);
  cout << "The time limit is: " << secondsOfSimuTime << endl;
  cout << "Number of elements is " << currentNumberOfElements << endl;
  time(&beginTime);
  timeInfo = localtime(&beginTime);  
  int targetNumberOfElements = context->getNumberOfElements();
  int connectTarget=0;
  int hydrophobicFlag   = context->getHydrophobicFlag();
  //float lowerBoundary   = context->getHydrophobicLower();
  //float topBoundary     = context->getHydrophobicTop();
  int diffusionFlag     = context->getDiffusionFlag();
  //int diffusionLevel    = context->getDiffusionLevel();
  float hPercentage;
  //float cPercentage;
  DLAElement* uniqueTarget;
  list<DLAElement*>  collideTarget;
  //float virtualTime = 0.0f;
  while(  currentNumberOfElements < targetNumberOfElements 
	   && elapsedSeconds < secondsOfSimuTime){
    activeElement = releaseTemplate->generateElement("CylinderElastin");
    activeElement->setSpeed(elementHeight*2.0f);
    //cout << activeElement->isActive() << endl;
    while(activeElement->isActive()){
      //time(&curTime);
      //cout << "---" <<endl;
      //cout << curTime-beginTime << endl;
      
      if(isOutOfBoundary(activeElement)){
        activeElement->stop();
        releaseTemplate->addToRecycleList(activeElement);
        break;        
      }      
      else{
        if(inCollidingArea(activeElement)){
          //cout << "In colliding area" << endl;
          connectTarget = aggregate->toBeCollided(activeElement, collideTarget);
          if(connectTarget>0){              
            if(connectTarget==1){
              list<DLAElement*>::iterator itr = collideTarget.begin();
              uniqueTarget = *itr;
            }
            else{
              //uniqueTarget = aggregate->getUniqueTarget(activeElement, collideTarget);
              activeElement->stop();
              releaseTemplate->addToRecycleList(activeElement);
              break;
            }
            //activeElement->stuckWith(uniqueTarget);
            uniqueTarget->attach(activeElement);

            if(hydrophobicFlag==1){
            	//cout << "here " << endl;
                float hydroWeight = aggregate->computeAffinityWeight2(activeElement, context->getDiffusionDistanceThreshold());
                hPercentage = hydroWeight/context->getSimuHydrophobicLength();
            	
              
              //hPercentage = aggregate->percentageOfOverlappedDomain(activeElement, uniqueTarget, 'h');
              //hPercentage = aggregate->percentageOfOverlapped(activeElement, uniqueTarget);
              //cout << "here 1" << endl;
              float probability;
              if(context->getHydroConstraintType()==1){
                probability = context->getLinearHydroProbability(hPercentage);
              }
              else if(context->getHydroConstraintType()==2){
                probability = context->getExponentialDensityHydroProbability(hPercentage);              	
              }
              else if(context->getHydroConstraintType()==3){
                probability = context->getExponentialCumulativeHydroProbability(hPercentage);              	
              }
              else if(context->getHydroConstraintType()==4){
                probability = context->getPowerHydroProbability(hPercentage);       
                //cout << probability << endl;
              }
              else if(context->getHydroConstraintType()==5){
                probability = context->getNormalHydroProbability(hPercentage);       
                //cout << probability << endl;
              }             
              else{
                  cout << "Constraint type " << context->getHydroConstraintType() << endl;            	  
            	  cout << "The value of Hydrophobic Constraint Type must be between [1:4]\n " << endl;
            	  return;
              }
              
              float vp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
              //if( hPercentage < 0.25f || vp>probability){
              if( vp>probability){            	  
                activeElement->stop();
                releaseTemplate->addToRecycleList(activeElement);
                break;
              }
              
              //float hydroWeight = aggregate->computeAffinityWeight2(activeElement, context->getDiffusionDistanceThreshold());
              //if(hydroWeight<6.0f){
              //    activeElement->stop();
              //    releaseTemplate->addToRecycleList(activeElement);
              //    break;            	  
              //}
            }
            
            //cout << "hPercentage: " << hPercentage << endl;
            //if(diffusionFlag==1 && aggregate->getNumberOfElements()>10){
            if(diffusionFlag==1 ){ // && aggregate->getNumberOfElements()>10){
       			aggregate->diffuseBasedOnEnergy3(uniqueTarget, activeElement);            	
            	//cout << diffusionLevel << endl;
/*            	if(diffusionLevel==3){
            		aggregate->cascadeDiffuse(uniqueTarget, activeElement, 3);
            	}
            	else if(diffusionLevel==2){ //level 2
            		//cout << "in" << endl;
            		//Spin based on favorable factors
           			aggregate->diffuseBasedOnEnergy2(uniqueTarget, activeElement, 2);
           			//cout << "out" << endl;
            	}
                else if(diffusionLevel==1){ //level 1
                	//Spin based on unfavorable factors
           			aggregate->diffuseBasedOnEnergy2(uniqueTarget, activeElement, 1);
                    //aggregate->diffuse(uniqueTarget, activeElement);
                }
*/                
            }
            else{
        	    float cAngle = uniqueTarget->getYZAngle(activeElement);
 	            uniqueTarget->addNeighbour(cAngle, activeElement);
 	            float tAngle = activeElement->getYZAngle(uniqueTarget);
	            activeElement->addNeighbour(tAngle, uniqueTarget);            	
            }
            activeElement->stop();
            aggregate->addElement(activeElement);
            if(context->getCenterAdjustment()==1){
              aggregate->adjustAggregateCenter();
            }
            
            maxX = aggregate->getMaxX() + halfElementWidth;
            minX = aggregate->getMinX() - halfElementWidth;
            maxY = aggregate->getMaxY() + halfElementHeight;
            minY = aggregate->getMinY() - halfElementHeight;
            maxZ = aggregate->getMaxZ() + halfElementDepth;
            minZ = aggregate->getMinY() - halfElementDepth;

            int curRemain = aggregate->getNumberOfElements()/100;
            if(curRemain > remains){
              cout << "Number of Elements in the aggreagte: " << curRemain * 100 << "\n";
              remains = curRemain;
            }
            //cout << "Number of Elements in the aggreagte: " << aggregate->getNumberOfElements() << endl;
            templateIsToExpand();
            break;
          }
          activeElement->setSpeed(halfElementHeight);
        }
      }
      activeElement->walk(0.5);
      //virtualTime += 0.5;
      //cout << activeElement->getX() << ", " << activeElement->getY() << ", " << activeElement->getZ() << endl;
    }
    currentNumberOfElements = aggregate->getNumberOfElements();
    time(&curTime);
    elapsedSeconds = curTime - beginTime; //difftime(curTime, beginTime);
    //cout << "Seconds elapsed: " << elapsedSeconds << endl;
  }
  
  //if(traceFileName!=""){
  //	traceFileName = "noshaking"+traceFileName;
  //  save(traceFileName.c_str());
  //}
  //else{
  //  save("noshakingtrace.xml");
  //}
  
  
  //aggregate->shaking(10000);
  
  timeInfo = localtime(&beginTime);  
  cout << "The simulation begins at " << asctime(timeInfo) << "\n";
  time(&endTime);  
  timeInfo = localtime(&endTime);  
  cout << "The simulation was done at " << asctime(timeInfo)<< "\n";
  cout << "It totally takes " << float(endTime - beginTime)/3600.0f << " hours." << "\n";
  cout << "" << "Computing the statistics of the aggregate ...... " << "\n";
  //Save the seconds spent into the aggregate.
  aggregate->setSecondsSpent(elapsedSeconds);
  //list<Element*>* pList = aggregate->getElements();
  //emit updateSimuView(pList);
/*  cout << "Aggregate width: " << aggregate->getMaxX() - aggregate->getMinX()
       << " height: "         << aggregate->getMaxY() - aggregate->getMinY()
       << " depth: "          << aggregate->getMaxZ() - aggregate->getMinZ() << endl;
  cout << "Computing the density, please wait ....................." << endl;
  float avgDensity = aggregate->computeAverageDensity(2.0f);
  cout << "The density is: " << avgDensity << endl;
  cout << "Computing the surface area, please wait ....................." << endl;
  float surfaceArea = aggregate->computeSurfaceArea(0.5f);
  cout << "The surface area is: " << surfaceArea << endl;
*/
  //cout << "Computing connectivity, please wait ....................." << endl;  
/*
  aggregate->computeAllNeighbours();
  sData.width = aggregate->getMaxX() - aggregate->getMinX();
  sData.height = aggregate->getMaxY() - aggregate->getMinY();
  sData.depth = aggregate->getMaxZ() - aggregate->getMinZ();
  sData.density = aggregate->computeAverageDensity(2.0f);
  sData.surfaceArea = aggregate->computeSurfaceArea(0.5f);
  sData.connectivity = aggregate->computeConnectivity();
*/
  //DataCollector dCollector(aggregate);
  
  dataCollector.setAggregate(aggregate);
  dataCollector.collectData();
  
  cout << dataCollector << endl;
  
  context->setTemplateWidth(releaseTemplate->getWidth());
  context->setTemplateHeight(releaseTemplate->getHeight());
  context->setTemplateDepth(releaseTemplate->getDepth());
  
  if(traceFileName!=""){
    save(traceFileName.c_str());
  }
  else{
    save("trace.xml");
  }
}

SimuContext* SimuModel::getContext(){
  return context;
}

Aggregate* SimuModel::getAggregate(){
  return aggregate;
}

void SimuModel::getStatisticData(StatisticData& data){
  data = sData;
}

int SimuModel::save(const char* fileName){
  ofstream outFile(fileName);
  if(!outFile.good()) {
    cerr << "Error while opening output file!" << endl;
    return -1;
  }    
  outFile << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  outFile << "<dla-trace>" << "\n";
  context->save(outFile);
  aggregate->save(outFile);
  outFile << dataCollector << "\n";
  //dataCollector.save(outFile);
  outFile << "</dla-trace>\n";
  outFile.close();
  return 1;
}

void SimuModel::setProfileName(string profileName){
  this->profileName = profileName;
}
