#ifndef SIMUMODEL_H
#define SIMUMODEL_H
#include <list>
#include <string>

#include "SimuContext.h"
#include "DLAReleaseTemplate.h"
#include "DLAElement.h"
#include "Aggregate.h"
#include "StatisticData.h"
#include "DataCollector.h"

using namespace std;

class SimuModel{
  public:
    SimuModel();
    // constructor for building models from model profiles
    SimuModel(string profileName, string traceFileName);
    // constructor for building models from trace files
    SimuModel(string traceFileName);
    
    ~SimuModel();
    //bool setup();
    void run();
    void setProfileName(string profileName);    
    SimuContext* getContext();
    Aggregate* getAggregate();
    void getStatisticData(StatisticData& data);
    string getProfilename();   
    int save(const char* fileName);
    void updateSimuView(list<DLAElement*>* pList);
    void addElement(DLAElement *);
    void templateChanged(float, float);
    void setAggregate(Aggregate*);

  private:
    bool            isOutOfBoundary(DLAElement* p);
    void            diffusing(DLAElement* p);
    bool            inCollidingArea(DLAElement* p);
    bool            templateIsToExpand();

    string                 profileName;
    string                 traceFileName;
    SimuContext*           context;
    DLAReleaseTemplate*    releaseTemplate;
    Aggregate*             aggregate;
    DLAElement*            activeElement;
    StatisticData          sData;
    DataCollector          dataCollector;

    float                  elementWidth;
    float                  halfElementWidth;
    float                  elementHeight;
    float                  halfElementHeight;
    float                  elementDepth;
    float                  halfElementDepth;
    float                  templateDistance;
    float                  twoTimesDistance;

    float                  maxX;
    float                  minX;
    float                  maxY;
    float                  minY;
    float                  maxZ;
    float                  minZ;
};

#endif
