#ifndef VECTOR2D_HPP_
#define VECTOR2D_HPP_

#include "Point2D.hpp"
#include <cmath>
#include <iostream>


//template <typename T>
//class Point2D;

template <typename T>
class Vector2D{
	
public:
	Vector2D(){
		m_x = T();
		m_y = T();
		m_id = 0;
	}
	
	Vector2D(const Point2D<T>& p2d, unsigned int id=0){
		m_x = p2d.x();
		m_y = p2d.y();
		m_id = id;
	}

	Vector2D(const Point2D<T>& p1, const Point2D<T>& p2, unsigned int id=0){
		m_x = p2.x() - p1.x();
		m_y = p2.y() - p1.y();
		m_id = id;
	}
	
	
	Vector2D(const T& xValue,  const T& yValue, unsigned int id=0){
		m_x = xValue;
		m_y = yValue;
		m_id = id;
	}
	
	T x() const{
		return m_x;
	}

	T y() const{
		return m_y;
	}

	void setPosition(T x, T y){
		m_x = x;
		m_y = y;
	}
	
	bool operator==(const Vector2D<T>& rhs){
		if( (m_x==rhs.x()) && (m_y==rhs.y())){
			return true;
		}
		return false;
	}
	
	Vector2D<T> operator+(const Vector2D<T>& rhs){
		Vector2D<T> tmpVec(m_x+rhs.x(), m_y+rhs.y());
		return tmpVec;
	}
	
	Vector2D<T>& operator+=(const Vector2D<T>& rhs){
		m_x += rhs.x();
		m_y += rhs.y();
		return (*this);
	}
	
	Vector2D<T> operator-(const Vector2D<T>& rhs){
		Vector2D<T> tmpVec(m_x-rhs.x(), m_y-rhs.y());
		return tmpVec;
	}
	
	Vector2D<T> operator-=(const Vector2D<T>& rhs){
		m_x -= rhs.x();
		m_y -= rhs.y();
		return (*this);
	}
	
	T dotProduct(const Vector2D<T>& rhs) const{
		return m_x*rhs.m_x + m_y*rhs.m_y;
	}	
	
	T length() const{
		return sqrt(m_x*m_x + m_y*m_y);
	}
	
	T angle() const{
		return acos(m_x/length());
	}
	
	T angleTo(const Vector2D<T>& rhs) const{
		T tmp = this->dotProduct(rhs);
		return acos(tmp/(this->length()*rhs.length()));
	}
	
	//For two 2D vectors U=(Ux, Uy), V(Vx, Vy),
	//U x V = Ux * Vy - Uy*Vx
	//The geometry meaning of the 2D product can be used to determine the relative
	//position of U and V, if U x V > 0, V is on the left hand side of U,
	//                     if U x V = 0, U and V are colinear
	//                     if U x V < 0, V is on the right hand side of U.
	T crossProduct(const Vector2D<T>& rhs) const {
		return (m_x*rhs.y() - m_y*rhs.x());
	}
private:
	T m_x;
	T m_y;
	unsigned int m_id;
};

template <typename T>
std::ostream& operator<<(std::ostream& ostr, const Vector2D<T>& vecValue){
	ostr << "(" << vecValue.x() << ", " << vecValue.y() << ")";
	return ostr;
}

#endif /*VECTOR2D_HPP_*/
