#ifndef XMLFILEMANAGER_H_
#define XMLFILEMANAGER_H_

#include "SimuContext.h"
#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <string>
#include <fstream>
#include <vector>
#include <exception>


//class SimuContext;
class Aggregate;

class XMLFileManager
{
public:
	XMLFileManager();
	virtual ~XMLFileManager();
	
	void loadTrace(const std::string& fileName, SimuContext* context, Aggregate* aggregate);
	void loadContext(const std::string& fileName, SimuContext* context);
	bool saveTrace(const std::string& fileName,  SimuContext* context, Aggregate* aggregate);
	void saveTrace(std::ofstream& outFile, SimuContext* context, Aggregate* aggregate);
	bool saveContext(const std::string& fileName, SimuContext* context);
	void saveContext(std::ofstream& outFile, SimuContext* context);
	void saveAggregate(std::ofstream& outFile, Aggregate* aggregate);
	
private:
	void loadAggregate(const xercesc::DOMElement* domElm, Aggregate* aggregate);	
	void loadContext(const xercesc::DOMElement* domElm, SimuContext* context);
	
	void loadContextParameters(const xercesc::DOMElement* contextRoot, SimuContext* context);
	void loadBasicContextParameters(const xercesc::DOMElement* contextRoot, SimuContext* context);	
	void loadCenterAdjustParameters(const xercesc::DOMElement* contextRoot, SimuContext* context);
	void loadDiffusionParameters(const xercesc::DOMElement* contextRoot, SimuContext* context);
	void loadHydroConstraintParameters(const xercesc::DOMElement* contextRoot, SimuContext* context);
	void loadParticleDomainParamenters(const xercesc::DOMElement* contextRoot, SimuContext* context);
	
	bool tagExists(const xercesc::DOMElement* root, const string& tagName);
	xercesc::DOMElement* getFirstChildElement(const xercesc::DOMElement* parent, const std::string& childName);
	void getStringParameter(const xercesc::DOMElement* root, const std::string& paraName, std::string& paraValue) throw(std::runtime_error);
	float getFloatParameter(const xercesc::DOMElement* root, const std::string& paraName);
	int getIntParameter(const xercesc::DOMElement* root, const std::string& paraName);	
	void parseElementTemplate(xercesc::DOMElement* dlaTemplate, std::vector<ElementTemplate>& tempVector);	
	xercesc::XercesDOMParser* XDomParser;
};

#endif /*TRACEMANAGER_H_*/
