###############################################################################
#
#Collect specific data from output files.
#Command line parameters:
#   target directory: at which directory, the output files will be processed.
#   file name pattern: what kind of files will be processed
#   data pattern: which data will be collected
#   output file: where to save the collected data
#
###############################################################################
#!/usr/bin/perl

use strict;
use File::stat;

if(!(scalar(@ARGV)>0) || scalar(@ARGV)<4){
  print "Usage:\n";
  print "perl collectSpecificStatisticData.pl \" targetDirectory fileNamePattern dataPattern outputFile\" \n";
  print "Example: \n";
  die "perl collectSpecificStatisticData.pl \"../../simulationdata/diffusion\" \"^output-try\(d*)\.xml\$\" \"^\\s*Density:\\s*(\\S*)\"  \"output.txt\"\n";  
}

my $targetDir = $ARGV[0];

if(!stat($targetDir)){
  die "\nError: \n\ttarget directory: '$targetDir' does not exist !\n";
}

open(OUTPUTFILE, "> $ARGV[3]");
&collectSpecificData($targetDir, $ARGV[1], $ARGV[2]);
close(OUTPUTFILE);

sub collectSpecificData(){
  my ($dirTarget, $filePattern, $dataPattern) = @_;
  my ($eachFile, @myDir);
  
  print $filePattern."\n";
  print $dataPattern."\n";
  opendir(TARGETDIR, $dirTarget);
  @myDir = readdir(TARGETDIR);
  closedir(TARGETDIR);
  my ($outputLine) = $dirTarget;
  my (%valueHash, $curValue);
  foreach(@myDir){
    $eachFile=$_;
    if($eachFile !~ /^\./){
      if(-d "$dirTarget/$eachFile"){
        my ($curDir) = $eachFile;
        print "DIR - $curDir\n";        
        &collectSpecificData("$dirTarget/$curDir", $filePattern, $dataPattern);
      }
      else{
        if($eachFile =~ /$filePattern/){
          print "$eachFile\n";        
          if(!defined($1)){
            die "No specific file pattern defined, please add \"()\" the interesting part in your file pattern.";
          }
          print $1."\n";
          $curValue=$1;
          
          open(TARGETFILE, "< $dirTarget/$eachFile");
          my($eachLine);
          my($found)=0;
          while(defined($eachLine = <TARGETFILE>)){
            #print $eachLine
            if($eachLine =~ /$dataPattern/){
              print $eachLine;
              if(!defined($1)){
                die "No specific data defined, please add \"()\" the interesting part in your data pattern.";
              }
              print "$1\n";
              $found=1;
              #$outputLine = $outputLine."\t".$1;
              $valueHash{$curValue}=$1;
            }
          }
          if($found==0){
            print "***------------ATTENTION-------------------***\n";
            print "No interested data found in file $dirTarget/$eachFile \n";
          }
          close(TARGETFILE);
        }
      }
    }
  }
  my(@keyList) = sort {$a <=> $b} keys(%valueHash);
  
  foreach(@keyList){
    print "-$_";
    $outputLine = $outputLine."\t".$valueHash{$_};
  }
  print "\n";
  print OUTPUTFILE $outputLine."\n";  
}

