# Generate network files that can be read by tyna from
# simulation traces.
#
# Command line parameters:
#   dirsource: the directory where simulation result files are located
#   dirtarget: the directory where generated files will be saved

#!/usr/bin/perl -w

use File::stat;
#use Number::Format;
use strict;


my ($dirSource, $dirTarget) = @ARGV;

#initialize the hash table
our %resHash = ();
our $numberOfRound = 0;

if(!defined($dirSource) || !defined($dirTarget)){
  die "Source or target directory is missed !\n\tUsage:\n\t\tperl generateNetworks.pl dirSoruce dirTarget\n";
}

if(!stat($dirSource)){
  die "\nError: \n\tSource directory: '$dirSource' does not exist !\n";
}

if(!stat($dirTarget)){
  if(!mkdir($dirTarget)){ # 1: succeeded, 0: failed.  
    die "\nError: \n\tMake target directory : '$dirTarget' failed !\n"
  }
}

&collectData($dirSource, $dirTarget);

my ($keyFile, $keyConn, $keyCenter, $tmpValue);

our %statHash = (); #Save the statistics data for each file


#print "Number of Round: $numberOfRound\n";
for $keyFile (sort mysort keys %resHash){
  #Compute New Length
  &compAverageAndDeviation($keyFile, "density", @{$resHash{$keyFile}{'density'}});
  &compAverageAndDeviation($keyFile, "surface_area", @{$resHash{$keyFile}{'surface_area'}});
  &compAverageAndDeviation($keyFile, "average_connectivity", @{$resHash{$keyFile}{'average_connectivity'}});
  &compAverageAndDeviation($keyFile, "average_length", @{$resHash{$keyFile}{'average_length'}});

  open(CONNFILE, ">$dirTarget/connectivity-$keyFile");  
  for $keyConn (sort {$a<=>$b} keys %{$resHash{$keyFile}{'connectivity'}}){
     #print "$keyConn  --- $resHash{$keyFile}{'connectivity'}{$keyConn}\n";
     $tmpValue = $resHash{$keyFile}{'connectivity'}{$keyConn}/$numberOfRound;
     print CONNFILE "$keyConn\t$tmpValue\n";
  }
  close(CONNFILE);

  open(CENTERFILE, ">$dirTarget/center-$keyFile");
  for $keyCenter (sort {$a<=>$b} keys %{$resHash{$keyFile}{'distance_center'}}){
     #print "$keyCenter  --- $resHash{$keyFile}{'distance_center'}{$keyCenter}\n";
     $tmpValue = $resHash{$keyFile}{'distance_center'}{$keyCenter}/$numberOfRound;
     print CENTERFILE "$keyCenter\t$tmpValue\n";
  }
  close(CENTERFILE);
}

my ($tmpAvg, $tmpDev, $fIndex);
$fIndex=0;
open(STATFILE, ">$dirTarget/stat_data.dat");
print STATFILE "##\tdensity\terror\tsurface\t\terror\t\tconn\terror\tlength\t\terror\n";
for $keyFile (sort mysort keys %statHash){
  print "$keyFile\n";
  $fIndex++;
  print STATFILE $fIndex;
  $tmpAvg = sprintf("%.3f", $statHash{$keyFile}{"density"}[0]);
  $tmpDev = sprintf("%.3f", $statHash{$keyFile}{"density"}[1]);
  print STATFILE "\t$tmpAvg\t$tmpDev";

  $tmpAvg = sprintf("%.3f", $statHash{$keyFile}{"surface_area"}[0]);
  $tmpDev = sprintf("%.3f", $statHash{$keyFile}{"surface_area"}[1]);
  print STATFILE "\t$tmpAvg\t$tmpDev";

  $tmpAvg = sprintf("%.3f", $statHash{$keyFile}{"average_connectivity"}[0]);
  $tmpDev = sprintf("%.3f", $statHash{$keyFile}{"average_connectivity"}[1]);
  print STATFILE "\t$tmpAvg\t$tmpDev";

  $tmpAvg = sprintf("%.3f", $statHash{$keyFile}{"average_length"}[0]);
  $tmpDev = sprintf("%.3f", $statHash{$keyFile}{"average_length"}[1]);
  print STATFILE "\t$tmpAvg\t$tmpDev\n";
}
close(STATFILE);
################################################################################
#
# Subroutines
#
################################################################################
sub mysort(){
  my ( $v1, $v2);
  $a=~/.*[\D](\d+).txt$/;
  $v1 = $1;
  $b=~/.*[\D](\d+).txt$/;
  $v2 = $1;
  $v1 <=> $v2;
}

sub compAverageAndDeviation(){
  my ($fileName, $itemName, @valueArray) = @_;
  my $arrLength = scalar(@valueArray);
  if($arrLength != $numberOfRound){
    print "$fileName -- $arrLength\n";
    die "Died in compAverageAndDeviation. The number of values is not equal to the number of simulations.";
  }
  my $aValue;
  my $arrSum = 0;
  my $avgValue = 0;
  my $deviation = 0;

  foreach $aValue (@valueArray){
    $arrSum += $aValue;
  }
  $avgValue = $arrSum/$arrLength;

  $arrSum = 0;
  foreach $aValue (@valueArray){
    $arrSum += ($aValue-$avgValue)*($aValue-$avgValue);
  }
  $deviation = sqrt($arrSum/$arrLength);
  #print "$fileName -- $itemName: $avgValue, $deviation\n";
  $statHash{$fileName}{$itemName} = [ $avgValue, $deviation ]
}

sub collectData(){
  my ($sourceDir, $targetDir)=@_;
  opendir(SOURCEALLDIR, $sourceDir);

  my ($eachDir);

  while(defined($eachDir = readdir(SOURCEALLDIR))){
    if($eachDir =~ /^round\d+$/){
      print $eachDir."\n";
      &collectDataOneRound($sourceDir."/".$eachDir, $targetDir);
      $numberOfRound++;
    }
  }
  close(SOURCEALLDIR);    
}

sub collectDataOneRound(){
  my ($sourceDir, $targetDir)=@_;
  print "$sourceDir  --- $targetDir\n";
  my ($eachFile, $eachLine, $curElm, $inConnectivity, $inCenterDistance );

  #if($sourceDir=~ /.*\/(round\d+)/){
  #  $roundSerial = $1;
  #}

  opendir(SOURCEDIR, $sourceDir);
  while(defined($eachFile = readdir(SOURCEDIR))){
    #print $eachFile."\n";
    if( $eachFile !~ /\.$/ && $eachFile =~ m/^output-(.*).txt$/i){
      #print $eachFile."\n";
      $inConnectivity=0;
      $inCenterDistance=0;
      open(RESULTFILE, $sourceDir."/".$eachFile);
      while(defined($eachLine = <RESULTFILE>)){
        if($eachLine =~ /^\s+Density:\s+(.*)/){
          if(!defined($resHash{$eachFile}{'density'})){
            #$resHash{$eachFile}{'density'} = $1;
            $resHash{$eachFile}{'density'} = [ $1 ];
          }
          else{
            #$resHash{$eachFile}{'density'} += $1; #Hashs of hash
            push(@{$resHash{$eachFile}{'density'}}, $1); #Hash of arrays of hashs
          }
          #print "Density: $resHash{$eachFile}{'density'}\n";
        }
        elsif($eachLine =~ /^\s+Surface Area:\s+(.*)/){
          if(!defined($resHash{$eachFile}{'surface_area'})){
            $resHash{$eachFile}{'surface_area'} = [ $1 ];
          }
          else{
            push(@{$resHash{$eachFile}{'surface_area'}}, $1);
          }

          #print "Surface Area: $1\n";
        }
        elsif($eachLine =~ /^\s+Average Connectivity:\s+(.*)/){
          if(!defined($resHash{$eachFile}{'average_connectivity'})){
            $resHash{$eachFile}{'average_connectivity'} = [ $1 ];
          }
          else{
            push(@{$resHash{$eachFile}{'average_connectivity'}}, $1);
          }

          #print "Average Connectivity: $1\n";
        }
        elsif($eachLine =~ /^\s+Aggregate Length:\s+(.*)/){
          if(!defined($resHash{$eachFile}{'average_length'})){
            $resHash{$eachFile}{'average_length'} = [ $1 ];
          }
          else{
            push(@{$resHash{$eachFile}{'average_length'}}, $1);
          }
          #print "Aggregate Length: $1\n";
        }
        elsif($eachLine =~ /^\s+Connectivity:/){
          #print "Connectivity:\n";
          $inConnectivity=1;
        }
        elsif($eachLine =~ /^\s+Distance to center:/){
          #print "Distance to center:\n";
          $inCenterDistance=1;
          $inConnectivity=0;
        }
        elsif($inConnectivity==1){
          if($eachLine =~ /^\t(\d+)\t+(\d+)/){
            if(!defined($resHash{$eachFile}{'connectivity'}{$1})){
              $resHash{$eachFile}{'connectivity'}{$1} = $2;
            }
            else{
              $resHash{$eachFile}{'connectivity'}{$1} += $2;
            }
            #print "$1 --- $2\n";
          }
          #print $eachLine;
        }
        elsif($inCenterDistance==1){
          if($eachLine =~ /^\t(\d+)\t+(\d+)/ || $eachLine =~ /^\t(-\d+)\t+(\d+)/){
            #print "$1 --- $2\n";
            #if($2>3){
              #print "$1 --- $2\n";
              if(!defined($resHash{$eachFile}{'distance_center'}{$1})){
                $resHash{$eachFile}{'distance_center'}{$1} = $2;
              }
              else{
                $resHash{$eachFile}{'distance_center'}{$1} += $2;
              }
            #}
            #else{
            #  print "$1 --- $2\n";
            #}
          }

          #print $eachLine;
        }
        else{
        }        
      }

      #recompute the aggregate length
      #my @centerKeys = keys(%{$resHash{$eachFile}{'distance_center'}});
      #my @sortedKeys = sort {$a<=>$b} @centerKeys;
      #print scalar(@sortedKeys)."\n";
      #print "$sortedKeys[0] --- $sortedKeys[scalar(@sortedKeys)-1]\n";
      #my $rLength = scalar(@{$resHash{$eachFile}{'average_length'}});
      #$resHash{$eachFile}{'average_length'}[$rLength - 1]=$sortedKeys[scalar(@sortedKeys)-1]-$sortedKeys[0];
      #

      close(RESULTFILE);
    }
  }
  closedir(SOURCEDIR);
}

