#include "CrossViewExt.h"
#include "GLCrossView.h"
#include "SimuContext.h"
#include "Aggregate.h"
#include <QtGui>

CrossViewExt::CrossViewExt(SimuContext* context, Aggregate* agg,  QWidget* parent)
  : QWidget(parent)
{
	QVBoxLayout* lay = new QVBoxLayout(this);
	QGLFormat fmt;
	fmt.setSampleBuffers(true);
	
	//GLCrossView* glView = new GLCrossView(context, agg, fmt, this);
	glView = new GLCrossView(context, agg, fmt, this);
	glView->resize(1000,1000);
	QGroupBox *groupBox = new QGroupBox(tr("Options"));
	QRadioButton* twoDCrossSurface = new QRadioButton("2D Cross Surface");
	twoDCrossSurface->setChecked(true);
	QRadioButton* twoDProjection = new QRadioButton("2D Projection");	
	QRadioButton* threeDSegment = new QRadioButton("3D Segment");	
	
	QCheckBox *showHullBox  = new QCheckBox(tr("Show Hull"));
	//showHullBox->setEnabled(false);
	
	QCheckBox *leftBox  = new QCheckBox(tr("Left"));
	leftBox->setEnabled(true);
	leftBox->setChecked(true);
	QString leftText;
	leftText.sprintf("%.3f", glView->leftPosition());
	QLineEdit *leftPosEdit = new QLineEdit(leftText);
	//leftPosEdit->setEnabled(false);
	
	QCheckBox *centerBox  = new QCheckBox(tr("Center"));
	centerBox->setChecked(true);
	centerBox->setEnabled(false);
	QString centerText;
	centerText.sprintf("%.3f", glView->centerPosition());	
	QLineEdit *centerPosEdit = new QLineEdit(centerText);
	centerPosEdit->setEnabled(false);
	
	QCheckBox *rightBox  = new QCheckBox(tr("Right"));
	rightBox->setEnabled(true);
	rightBox->setChecked(true);
	QString rightText;
	rightText.sprintf("%.3f", glView->rightPosition());	
	QLineEdit *rightPosEdit = new QLineEdit(rightText);
	//rightPosEdit->setEnabled(false);
	
	QLabel    *scaleLabel = new QLabel(tr("Scale: "));
	QString scaleText;
	scaleText.sprintf("%.3f", glView->scale());	
	QLineEdit *scaleEdit = new QLineEdit(scaleText);
	
	QPushButton* pButton = new QPushButton("&Export Image", this);
	
	QGridLayout *gLay = new QGridLayout();
	gLay->addWidget(twoDCrossSurface, 0, 0);
	gLay->addWidget(twoDProjection, 0, 1);
	gLay->addWidget(threeDSegment, 0, 2);
	gLay->addWidget(showHullBox, 0, 3); //, 1, 2);	
	gLay->addWidget(scaleLabel, 0, 4);
	gLay->addWidget(scaleEdit, 0, 5);	
	
	gLay->addWidget(leftBox, 1, 0);
	gLay->addWidget(leftPosEdit, 1, 1);
	gLay->addWidget(centerBox, 1, 2);
	gLay->addWidget(centerPosEdit, 1, 3);
	gLay->addWidget(rightBox, 1, 4);
	gLay->addWidget(rightPosEdit, 1, 5);
	
	gLay->addWidget(pButton, 2, 0);
	groupBox->setLayout(gLay);
	
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), glView, 	   	SLOT(set2DCrossSurfaceView(bool)));
/*	connect(twoDCrossSurface, SIGNAL(clicked(bool)), showHullBox,  	SLOT(setEnabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), centerBox,  	SLOT(setEnabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), centerPosEdit, SLOT(setEnabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), leftBox,      	SLOT(setDisabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), leftPosEdit,  	SLOT(setDisabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), rightBox,     	SLOT(setDisabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), rightPosEdit, 	SLOT(setDisabled(bool)));	
*/
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), showHullBox,   	SLOT(setDisabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), centerBox,     	SLOT(setDisabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), centerPosEdit, 	SLOT(setDisabled(bool)));	
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), leftBox,       	SLOT(setEnabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), leftBox,       	SLOT(setChecked(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), leftPosEdit,   	SLOT(setEnabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), rightBox,      	SLOT(setEnabled(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), rightBox,      	SLOT(setChecked(bool)));
	connect(twoDCrossSurface, SIGNAL(clicked(bool)), rightPosEdit,  	SLOT(setEnabled(bool)));
	
	connect(threeDSegment, SIGNAL(clicked(bool)), glView, 			SLOT(set3DSegmentView(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), showHullBox, 		SLOT(setDisabled(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), centerBox, 		SLOT(setDisabled(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), centerPosEdit, 	SLOT(setDisabled(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), leftBox,       	SLOT(setEnabled(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), leftBox,       	SLOT(setChecked(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), leftPosEdit,   	SLOT(setEnabled(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), rightBox,      	SLOT(setEnabled(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), rightBox,      	SLOT(setChecked(bool)));
	connect(threeDSegment, SIGNAL(clicked(bool)), rightPosEdit,  	SLOT(setEnabled(bool)));
	
	connect(twoDProjection, SIGNAL(clicked(bool)), glView, 	   	  	SLOT(set2DProjectionView(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), showHullBox,   	SLOT(setDisabled(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), centerBox,     	SLOT(setDisabled(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), centerPosEdit, 	SLOT(setDisabled(bool)));	
	connect(twoDProjection, SIGNAL(clicked(bool)), leftBox,       	SLOT(setEnabled(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), leftBox,       	SLOT(setChecked(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), leftPosEdit,   	SLOT(setEnabled(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), rightBox,      	SLOT(setEnabled(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), rightBox,      	SLOT(setChecked(bool)));
	connect(twoDProjection, SIGNAL(clicked(bool)), rightPosEdit,  	SLOT(setEnabled(bool)));

	
	connect(leftBox, SIGNAL(clicked(bool)), glView, SLOT(setLeftChecker(bool)));
	connect(leftPosEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setLeftPosition(const QString&)) );
	connect(centerBox, SIGNAL(clicked(bool)), glView, SLOT(setCenterChecker(bool)));
	connect(centerPosEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setCenterPosition(const QString&)) );	
	connect(rightBox, SIGNAL(clicked(bool)), glView, SLOT(setRightChecker(bool)));
	connect(rightPosEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setRightPosition(const QString&)) );
	connect(glView, SIGNAL(scaleChanged(const QString&)), scaleEdit, SLOT(setText(const QString&)));
	connect(scaleEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setScale(const QString&)));
	connect(showHullBox, SIGNAL(clicked(bool)), glView, SLOT(setShowConvexHull(bool)));
	
	connect(pButton, SIGNAL(clicked(bool)), this, SLOT(exportImage()));
	lay->addWidget(glView);
	lay->addWidget(groupBox);
	setLayout(lay);
	//resize(1000,1000);
}

CrossViewExt::~CrossViewExt()
{
}

bool CrossViewExt::exportImage(){
	QString fileName = QFileDialog::getSaveFileName(this);
	if(fileName.isEmpty()){
		return false;
	}
	QPixmap pixmap = glView->renderPixmap();
	
	return pixmap.save(fileName);	
}