#include "DetailViewExt.h"
#include "GLDetailView.h"
#include "SimuContext.h"
#include "Aggregate.h"
#include <QtGui>

DetailViewExt::DetailViewExt(SimuContext* context, Aggregate* agg, QWidget* parent)
  : QWidget(parent)
{
	QVBoxLayout* lay = new QVBoxLayout(this);
	GLDetailView* glView = new GLDetailView(context, agg, this);
	
	QGroupBox *groupBox = new QGroupBox(tr("Options"), this);
	QRadioButton* twoDimension = new QRadioButton("2D View");
	twoDimension->setChecked(true);
	QRadioButton* threeDimension = new QRadioButton("3D View");

	QCheckBox *cutCheckbox = new QCheckBox("Cut");
	cutCheckbox->setEnabled(false);

	QLabel    *idLabel = new QLabel(tr("Particle ID: "));	
	QString   idText;
	idText.sprintf("%d", glView->particleId());
	QLineEdit *idEdit = new QLineEdit(idText);
	
	QLabel    *distLabel = new QLabel(tr("Distance Threshold: "));	
	QString   distText;
	distText.sprintf("%f", glView->distThreshold());
	QLineEdit *distEdit = new QLineEdit(distText);
	
	QLabel    *scaleLabel = new QLabel(tr("Scale: "));
	QString scaleText;
	scaleText.sprintf("%f", glView->scale());	
	QLineEdit *scaleEdit = new QLineEdit(scaleText);
	
	QGridLayout *gLay = new QGridLayout();
	gLay->addWidget(twoDimension, 0, 0, 1, 2);
	gLay->addWidget(threeDimension, 0, 2, 1, 2);
	gLay->addWidget(cutCheckbox, 0, 4, 1, 2);
	gLay->addWidget(idLabel, 1, 0);
	gLay->addWidget(idEdit, 1, 1);
	gLay->addWidget(distLabel, 1, 2);
	gLay->addWidget(distEdit, 1, 3);	
	gLay->addWidget(scaleLabel, 1, 4);
	gLay->addWidget(scaleEdit, 1, 5);	
	
	groupBox->setLayout(gLay);
	
	connect(twoDimension, SIGNAL(clicked(bool)), glView, SLOT(setTwoDimension(bool)));
	connect(threeDimension, SIGNAL(clicked(bool)), glView, SLOT(setThreeDimension(bool)));
	connect(threeDimension, SIGNAL(toggled(bool)), cutCheckbox, SLOT(setEnabled(bool)));
	connect(cutCheckbox, SIGNAL(stateChanged(int)), glView, SLOT(setThreeDCut(int)));	
	connect(idEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setParticleId(const QString&)));	
	connect(distEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setDistThreshold(const QString&)));
	connect(glView, SIGNAL(scaleChanged(const QString&)), scaleEdit, SLOT(setText(const QString&)));	
	connect(scaleEdit, SIGNAL(textEdited(const QString&)), glView, SLOT(setScale(const QString&)));
	
	lay->addWidget(glView);
	lay->addWidget(groupBox);
	setLayout(lay);
}

DetailViewExt::~DetailViewExt()
{
}
