#include <QtGui>
#include <QtOpenGL>
//#include <QGroupBox>
#include <cmath>
#include "GLCrossView.h"
#include "Aggregate.h"
#include "SimuContext.h"
#include "DLAElement.h"
#include "visualutils.h"

//#include <iostream>

GLCrossView::GLCrossView(SimuContext* pContext, Aggregate* pAgg, const QGLFormat& format, QWidget* parent)
  :QGLWidget(format, parent), m_xRot(0), m_yRot(0), m_zRot(0), m_leftChecked(false), m_leftPosition(-30.0f)
  , m_centerChecked(true), m_centerPosition(0.0f), m_rightChecked(false), m_rightPosition(30.0f), m_scale(1.0f)
  , m_2DCrossSurfaceView(true), m_2DProjectionView(false), m_3DSegmentView(false), m_showConvexHull(false){
  if(pContext!=NULL && pAgg!=NULL){
    m_simuAggregate = pAgg;
    m_simuContext   = pContext;
	m_simuAggregate->getElementsAtXFromMap(m_leftPosition, m_leftList);
	m_simuAggregate->getElementsAtXFromMap(m_centerPosition, m_centerList);
	m_simuAggregate->getElementsAtXFromMap(m_rightPosition, m_rightList);
  }
  else{
    m_simuAggregate = NULL;
    m_simuContext   = NULL;
  }
  //resize(800,800);
}

GLCrossView::~GLCrossView(){
}

QSize GLCrossView::minimumSizeHint() const{
  return QSize(50, 50);
}

QSize GLCrossView::sizeHint() const{
  return QSize(1000, 1000);
}

float GLCrossView::leftPosition() const{
	return m_leftPosition;
}

float GLCrossView::centerPosition() const{
	return m_centerPosition;
}

float GLCrossView::rightPosition() const{
	return m_rightPosition;
}

float GLCrossView::scale() const{
	return m_scale;
}

void GLCrossView::initializeGL()
{
/*    // Light values and coordinates
    GLfloat  ambientLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat  diffuseLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat lightPos[] = { 0.0f, -20.0f, -20.0f, 1.0f };
    GLfloat  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
    //GLfloat  specref[] = { 0.0f, 0.0f, 1.0f, 1.0f };
    
    //buildList();
    glEnable(GL_DEPTH_TEST);	// Hidden surface removal
    glEnable(GL_LIGHTING);
    
    // Setup and enable light 0
    glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT0,GL_SPECULAR, specular);    
    glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    glEnable(GL_LIGHT0);
*/
    // Light values and coordinates
    GLfloat  ambientLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat  diffuseLight[] = { 0.1f, 0.2f, 0.1f, 1.0f };
    GLfloat lightPos[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat lightPos1[] = { 0.0f, -20.0f, 0.0f, 1.0f };
    GLfloat lightPos2[] = { 0.0f, 20.0f, 0.0f, 1.0f };    
    GLfloat  specular[] = { 0.5f, 0.5f, 0.5f, 1.0f };
    glEnable(GL_DEPTH_TEST);	// Hidden surface removal
    //glShadeModel(GL_SMOOTH);
    glEnable(GL_LIGHTING);

    // Setup and enable light 0
    //glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
    //glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
    //glLightfv(GL_LIGHT0,GL_SPECULAR, specular);    
    //glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    //glEnable(GL_LIGHT0);

    //glLightfv(GL_LIGHT1,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT1,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT1,GL_SPECULAR, specular);    
    glLightfv(GL_LIGHT1,GL_POSITION,lightPos1);
    glEnable(GL_LIGHT1);

    //glLightfv(GL_LIGHT2,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT2,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT2,GL_SPECULAR, specular);    
    glLightfv(GL_LIGHT2,GL_POSITION,lightPos2);
    glEnable(GL_LIGHT2);
        
    // Enable color tracking
    glEnable(GL_COLOR_MATERIAL);
	
    // Set Material properties to follow glColor values
    glColorMaterial(GL_FRONT, GL_SPECULAR); //AMBIENT_AND_DIFFUSE);
    //glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    // All materials hereafter have full specular reflectivity
    // with a high shine
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
    //glMateriali(GL_FRONT, GL_SHININESS, 256);
    
    // Light blue background
    glClearColor(1.0f, 1.0f, 1.0f, 1.0f );
    // Black background
    //glClearColor(1.0f, 1.0f, 1.0f, 1.0f );

    // Set drawing color to green
    glColor3f(0.0f, 1.0f, 0.0f);
}

void GLCrossView::paintGL(){
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glLoadIdentity();
  glPushMatrix();
  //Antialiasing
  //glEnable(GL_BLEND);
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //glEnable(GL_POINT_SMOOTH);
  //glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
  //glEnable(GL_LINE_SMOOTH);
  //glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  //glEnable(GL_POLYGON_SMOOTH);
  //glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
  
  glTranslated(0.0, 0.0, -100.0);
  glRotated(m_xRot / 16.0, 1.0, 0.0, 0.0);
  glRotated(m_yRot / 16.0, 0.0, 1.0, 0.0);
  glRotated(m_zRot / 16.0, 0.0, 0.0, 1.0);
  glScaled(m_scale,m_scale,m_scale);
  if(m_2DCrossSurfaceView){
	  draw2DCrossSurface();
  }
  if(m_2DProjectionView){
	  draw2DProjection();
  }
  if(m_3DSegmentView){
	  draw3DSegment();
  }
  glPopMatrix();
}

void GLCrossView::draw2DCrossSurface(){
  glDisable(GL_LIGHTING);	
  glPushMatrix();
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  std::list<DLAElement*>::iterator itr;
  if(m_simuAggregate!=NULL && m_simuContext!=NULL){
	 if(m_showConvexHull){  
		glPushMatrix();
		list<DLAElement*> hullList;	
		m_simuAggregate->getConvexHullPoints(m_centerPosition, hullList);
		list<DLAElement*>::iterator hItr;
		glColor3f(0.0f, 0.0f, 0.0f);	
		for(hItr=hullList.begin(); hItr!=hullList.end(); hItr++){
			//cout << (*hItr) << endl;
			glPushMatrix();
			glTranslated((*hItr)->getZ(), (*hItr)->getY(), 0.0f); //m_rightPosition); //(*itr)->getX());
			gluDisk(pObj, 0, m_simuContext->getElementHeight()/2.0, 26, 26);
			glPopMatrix();		  
		}
	
		glBegin(GL_LINE_LOOP);
		for(hItr=hullList.begin(); hItr!=hullList.end(); hItr++){
			glVertex3f((*hItr)->getZ(), (*hItr)->getY(), 0.0f); //m_rightPosition); //(*itr)->getX());
		}
		glEnd();
		glPopMatrix();
	 }
		  
	 list<DLAElement*>* pElms;
	 list<DLAElement*>::const_iterator eItr;
	 pElms = m_simuAggregate->getElements();
	 for(eItr=pElms->begin(); eItr!=pElms->end(); eItr++){
	 if(  ((*eItr)->getX()+m_simuContext->getElementWidth()/2.0f)>=m_leftPosition
	   && ((*eItr)->getX()-m_simuContext->getElementWidth()/2.0f)<=m_rightPosition
	   ){
		 glPushMatrix();
		 glColor3f(1.0f, 0.0f, 0.0f);				 
		 glTranslated((*eItr)->getZ(), (*eItr)->getY(), 0.0f); //m_leftPosition); //(*itr)->getX());
		 gluDisk(pObj, 0, m_simuContext->getElementHeight()/2.0, 26, 26);
		 glPopMatrix();				  
	  }
	 }
	 
	 //glDisable(GL_LIGHTING);
	 //glDisable(GL_DEPTH_TEST);	  
	 //glEnable(GL_BLEND);
	 //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
/*	 
	 if(m_centerChecked){
		//glColor4f(0.0f, 1.0f, 0.0f, 0.5f);
		glColor3f(0.0f, 0.0f, 1.0f);
		for(itr=m_centerList.begin(); itr!=m_centerList.end(); itr++){
			glPushMatrix();
			glTranslated((*itr)->getZ(), (*itr)->getY(), 0.0f); //m_centerPosition);
			gluDisk(pObj, 0, m_simuContext->getElementHeight()/2.0, 13, 13);
			glPopMatrix();
		}		 
	 }
*/ 
     //glDisable(GL_BLEND);
	 //glEnable(GL_LIGHTING);
	 //glEnable(GL_DEPTH_TEST);
  }
  gluDeleteQuadric(pObj);
  glPopMatrix();
}

void GLCrossView::draw2DProjection(){
	  glDisable(GL_LIGHTING);	
	  glPushMatrix();
	  GLUquadricObj* pObj;
	  pObj = gluNewQuadric();
	  std::list<DLAElement*>::iterator itr;
	  if(m_simuAggregate!=NULL && m_simuContext!=NULL){
		 glDisable(GL_LIGHTING);
		 glDisable(GL_DEPTH_TEST);	  
		 glEnable(GL_BLEND);
		 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		  
		 list<DLAElement*>* pElms;
		 list<DLAElement*>::const_iterator eItr;
		 pElms = m_simuAggregate->getElements();
		 for(eItr=pElms->begin(); eItr!=pElms->end(); eItr++){
		 if(  ((*eItr)->getX()+m_simuContext->getElementWidth()/2.0f)>=m_leftPosition
		   && ((*eItr)->getX()-m_simuContext->getElementWidth()/2.0f)<=m_rightPosition
		   ){
			  
			 glPushMatrix();
			 glColor4f(1.0f, 0.0f, 0.0f, 0.5f);				 
			 glTranslated((*eItr)->getZ(), (*eItr)->getY(), 0.0f); //m_leftPosition); //(*itr)->getX());
			 gluDisk(pObj, 0, m_simuContext->getElementHeight()/2.0, 13, 13);
			 glPopMatrix();				  
		  }
		 }
	     glDisable(GL_BLEND);
		 glEnable(GL_LIGHTING);
		 glEnable(GL_DEPTH_TEST);		  
	 }
	 gluDeleteQuadric(pObj);
	 glPopMatrix();
}

void GLCrossView::draw3DSegment(){
  glEnable(GL_LIGHTING);	
  glPushMatrix();
  glTranslatef(0.0f, 0.0f, -(m_leftPosition+m_rightPosition)/2.0f);

  if(m_simuAggregate!=NULL && m_simuContext!=NULL){
	 std::list<DLAElement*>* eList = m_simuAggregate->getElements();
	 std::list<DLAElement*>::const_iterator itr;
	 for(itr=eList->begin(); itr!=eList->end(); itr++){
		 if(  ((*itr)->getX()+m_simuContext->getElementWidth()/2.0f)>=m_leftPosition
		   && ((*itr)->getX()-m_simuContext->getElementWidth()/2.0f)<=m_rightPosition
		   ){
		 glPushMatrix();
	     glTranslatef((*itr)->getZ(), (*itr)->getY(), (*itr)->getX());
	     drawAllTropoelastin( (*itr)->getX()
		    				, m_leftPosition  //(*itr)->getX() - 10.0f //m_leftPosition
		    				, m_rightPosition //(*itr)->getX() + 10.0f //m_rightPosition		    						 
		    				, m_simuContext->getElementWidth()
		    				, m_simuContext->getElementHeight()/2.0
		    				, *(m_simuContext->getElementTemplate())
		    				, (*itr)->getOrientation()
		    				);		    	
		 glPopMatrix();
		 }
	 }
  }
  glPopMatrix();
}

void GLCrossView::resizeGL(int w, int h){
  glViewport(0, 0, w, h);
  
  GLfloat fAspect = (GLfloat)w/(GLfloat)h;
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0f, fAspect, 20.0f, 200.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void GLCrossView::setXRotation(int angle){
  normalizeAngle(&angle);
  if(angle!=m_xRot){
	  m_xRot = angle;
	  emit xRotationChanged(angle);
	  updateGL();
  }
}

void GLCrossView::setYRotation(int angle){
  normalizeAngle(&angle);
  if(angle!=m_yRot){
	  m_yRot = angle;
	  emit yRotationChanged(angle);
	  updateGL();
  }
}

void GLCrossView::setZRotation(int angle){
  normalizeAngle(&angle);
  if(angle!=m_zRot){
	  m_zRot = angle;
	  emit zRotationChanged(angle);
	  updateGL();
  }
}

void GLCrossView::setShowConvexHull(bool value){
	if(m_showConvexHull!=value){
		m_showConvexHull = value;
		updateGL();
	}
}

void GLCrossView::set2DCrossSurfaceView(bool value){
	if(m_2DCrossSurfaceView!=value){
		m_2DCrossSurfaceView = value;
		m_2DProjectionView = !value;
		m_3DSegmentView = !value;
		updateGL();
	}
}

void GLCrossView::set2DProjectionView(bool value){
	if(m_2DProjectionView!=value){
		m_2DProjectionView = value;
		m_2DCrossSurfaceView = !value;		
		m_3DSegmentView = !value;
		updateGL();
	}	
}

void GLCrossView::set3DSegmentView(bool value){
	if(m_3DSegmentView!=value){
		m_3DSegmentView = value;
		m_2DCrossSurfaceView = !value;
		m_2DProjectionView   = !value;
		updateGL();
	}
}

void GLCrossView::setLeftChecker(bool checked){
	if(checked != m_leftChecked){
		m_leftChecked = checked;
		emit leftCheckerChanged(checked);
		//std::cerr << "Left Checker" << std::endl;
		updateGL();
	}
}

void GLCrossView::setLeftPostion(float position){
	if(position!=m_leftPosition){
	  if(!m_leftList.empty()){
		  m_leftList.clear();
	  }
	  m_leftPosition = position;
	  m_simuAggregate->getElementsAtXFromMap(position, m_leftList);
	  emit leftPositionChanged(position);
	  QString str;
	  str.sprintf("%.3f", position);
	  emit leftPositionChanged(str);
	  updateGL();
	}
}

void GLCrossView::setLeftPosition(const QString&  str){
	float position = str.toFloat();
	if(position!=m_leftPosition){
	  if(!m_leftList.empty()){
		  m_leftList.clear();
	  }
	  m_leftPosition = position;
	  m_simuAggregate->getElementsAtXFromMap(position, m_leftList);
	  emit leftPositionChanged(position);
	  emit leftPositionChanged(str);
	  updateGL();
	}
}

void GLCrossView::setCenterChecker(bool checked){
	if(checked != m_centerChecked){
		m_centerChecked = checked;
		emit centerCheckerChanged(checked);
		updateGL();
	}
}

void GLCrossView::setCenterPosition(float position){
	if(position!=m_centerPosition){
		if(!m_centerList.empty()){
			m_centerList.clear();
		}
		m_centerPosition = position;
		m_simuAggregate->getElementsAtXFromMap(position, m_centerList);
		emit centerPositionChanged(position);
	    QString str;
		str.sprintf("%.3f", position);
		emit centerPositionChanged(str);
		
		updateGL();
	}
}

void GLCrossView::setCenterPosition(const QString&  str){
	float position = str.toFloat();
	if(position!=m_centerPosition){
	  if(!m_centerList.empty()){
		  m_centerList.clear();
	  }
	  m_centerPosition = position;
	  m_simuAggregate->getElementsAtXFromMap(position, m_centerList);
	  emit centerPositionChanged(position);
	  emit centerPositionChanged(str);
	  updateGL();
	}
}

void GLCrossView::setRightChecker(bool checked){
	if(checked != m_rightChecked){
		m_rightChecked = checked;
		emit rightCheckerChanged(checked);
		updateGL();
	}
}

void GLCrossView::setRightPosition(float position){
	if(position!=m_rightPosition){
		if(!m_rightList.empty()){
			m_rightList.clear();
		}
		m_rightPosition = position;
		m_simuAggregate->getElementsAtXFromMap(position, m_rightList);
		emit rightPositionChanged(position);
		QString str;
		str.sprintf("%.3f", position);
		emit rightPositionChanged(str);
		
		updateGL();
	}
}


void GLCrossView::setRightPosition(const QString&  str){
	float position = str.toFloat();
	if(position!=m_rightPosition){
	  if(!m_rightList.empty()){
		  m_rightList.clear();
	  }
	  m_rightPosition = position;	  
	  m_simuAggregate->getElementsAtXFromMap(position, m_rightList);
	  emit rightPositionChanged(position);
	  emit rightPositionChanged(str);
	  updateGL();
	}
}

void GLCrossView::setScale(float value){
	if(value!=m_scale){		
		m_scale = value;
		emit scaleChanged(value);

		QString str;
		str.sprintf("%.3f", value);
		emit scaleChanged(str);
		
		updateGL();
	}
}


void GLCrossView::setScale(const QString& str){
	float value = str.toFloat();
	if(value!=m_scale){
		m_scale = value;
		
		emit scaleChanged(str);
		emit scaleChanged(value);

		updateGL();
	}
}

void GLCrossView::mousePressEvent(QMouseEvent* event){
	m_lastPos = event->pos();
}

void GLCrossView::mouseMoveEvent(QMouseEvent* event){
	int dx = event->x() - m_lastPos.x();
	int dy = event->y() - m_lastPos.y();
	
	if(event->buttons() & Qt::LeftButton){
		setXRotation(m_xRot + 8 * dy);
		setYRotation(m_yRot + 8 * dx);
	}
	else if(event->buttons() & Qt::RightButton){
		setXRotation(m_xRot + 8 * dy);
		setZRotation(m_zRot + 8 * dx);
	}
	m_lastPos = event->pos();
}

void GLCrossView::mouseDoubleClickEvent(QMouseEvent* event){
	if(event->buttons() & Qt::LeftButton){
		setScale(m_scale * 2);
	}
	else if(event->buttons() & Qt::RightButton){
		setScale(m_scale * 0.5);
	}
}

void GLCrossView::normalizeAngle(int* angle){
	while(*angle < 0){
		*angle += 360 * 16;
	}
	while(*angle > 360 * 16){
		*angle -= 360 * 16;
	}
}
