#ifndef GLCROSSVIEW_H_
#define GLCROSSVIEW_H_

#include <QGLWidget>
#include <QGLFormat>
#include <list>

class Aggregate;
class SimuContext;
class DLAElement;
class QGroupBox;

class GLCrossView : public QGLWidget
{
	Q_OBJECT
	
public:
	GLCrossView(SimuContext* pContext, Aggregate* pAgg, const QGLFormat& format, QWidget* parent = 0);
	virtual ~GLCrossView();
	QSize minimumSizeHint() const;
	QSize sizeHint() const;
	float leftPosition() const;
	float centerPosition() const;
	float rightPosition() const;
	float scale() const;
	
protected:
	void initializeGL();
	void paintGL();
	void resizeGL(int width, int height);
	void mousePressEvent(QMouseEvent* event);
	void mouseMoveEvent(QMouseEvent* event);
	void mouseDoubleClickEvent(QMouseEvent* event);
	void draw2DCrossSurface();
	void draw2DProjection();
	void draw3DSegment();
	
public slots:
    void setXRotation(int angle);
    void setYRotation(int angle);
    void setZRotation(int angle);
    void setLeftChecker(bool checked);
    void setLeftPostion(float position);
    void setLeftPosition(const QString& str);
    void setCenterChecker(bool checked);
    void setCenterPosition(float position);
    void setCenterPosition(const QString& str);
    void setRightChecker(bool checked);
    void setRightPosition(float position);
    void setRightPosition(const QString& str);
    void setScale(float value);
    void setScale(const QString& str);
    void set2DCrossSurfaceView(bool value);
    void set2DProjectionView(bool value);    
    void set3DSegmentView(bool value);
    void setShowConvexHull(bool value);
signals:
    void xRotationChanged(int angle);
    void yRotationChanged(int angle);
    void zRotationChanged(int angle);
    void leftCheckerChanged(bool checked);
    void leftPositionChanged(float position);
    void leftPositionChanged(const QString& str);
    void centerCheckerChanged(bool checked);
    void centerPositionChanged(float position);
    void centerPositionChanged(const QString& str);
    void rightCheckerChanged(bool checked);
    void rightPositionChanged(float position);
    void rightPositionChanged(const QString& str);    
    void scaleChanged(float value);
    void scaleChanged(const QString& str);
    void showConvexHullChanged(bool value);
    
private:
	void   normalizeAngle(int* angle);
	//QGroupBox* createOptionGroup();
	int    m_xRot;
	int    m_yRot;
	int    m_zRot;	
	QPoint m_lastPos;
	
	bool   m_leftChecked;
	float  m_leftPosition;	
	bool   m_centerChecked;
	float  m_centerPosition;
	bool   m_rightChecked;
	float  m_rightPosition;
	float  m_scale;
	bool   m_2DCrossSurfaceView;
	bool   m_2DProjectionView;
	bool   m_3DSegmentView;
	bool   m_showConvexHull;
	
	Aggregate*   m_simuAggregate;
	SimuContext* m_simuContext;
	std::list<DLAElement*> m_centerList;
	std::list<DLAElement*> m_leftList;
	std::list<DLAElement*> m_rightList;
	
	//QGroupBox*   m_optBox;
};

#endif /*GLCROSSVIEW_H_*/
