#include <QtGui>
#include <QtOpenGL>
//#include <QGroupBox>
#include <cmath>
#include "GLDetailView.h"
#include "Aggregate.h"
#include "SimuContext.h"
#include "DLAElement.h"
#include "visualutils.h"

//#include <iostream>

GLDetailView::GLDetailView(SimuContext* pContext, Aggregate* pAgg, QWidget* parent)
  :QGLWidget(parent), m_xRot(0), m_yRot(0), m_zRot(0)
  , m_scale(0.5f), m_particleId(0), m_distThreshold(1.0f), m_layersToShow(1)
  , m_twoDimension(true), m_threeDimension(false), m_cut(false){
  if(pContext!=NULL && pAgg!=NULL){
    m_simuAggregate = pAgg;
    m_simuContext   = pContext;
  }
  else{
    m_simuAggregate = NULL;
    m_simuContext   = NULL;
  }
}

GLDetailView::~GLDetailView(){
}

QSize GLDetailView::minimumSizeHint() const{
  return QSize(50, 50);
}

QSize GLDetailView::sizeHint() const{
  return QSize(800, 800);
}

float GLDetailView::scale() const{
	return m_scale;
}

int GLDetailView::particleId() const{
	return m_particleId;
}

float GLDetailView::distThreshold() const{
	return m_distThreshold;
}

int GLDetailView::layersToShow() const{
	return m_layersToShow;
}

void GLDetailView::setScale(float value){
	if(value!=m_scale){
	  m_scale = value;
	  updateGL();	  
	  emit scaleChanged(value);
	  QString str = QString::number(value);
	  emit scaleChanged(str);
	}
}

void GLDetailView::setScale(const QString& str){
	float value = str.toFloat();
	if(value!=m_scale){
	  m_scale = value;
	  updateGL();	  
	  emit scaleChanged(value);
	  emit scaleChanged(str);
	}
}

void GLDetailView::setParticleId(int value){
	if(value!=m_particleId){
		m_particleId = value;
		updateGL();
		emit particleIdChanged(value);
	}
}

void GLDetailView::setParticleId(const QString& str){
	int value = str.toInt();
	setParticleId(value);	
}

void GLDetailView::setDistThreshold(float value){
	if(value!=m_distThreshold){
		m_distThreshold = value;
		updateGL();
		emit distThresholdChanged(value);
	}
}

void GLDetailView::setDistThreshold(const QString& str){
	float value = str.toFloat();
	setDistThreshold(value);
}

void GLDetailView::setLayersToShow(int value){
	if(value!=m_layersToShow){
		m_layersToShow = value;
		updateGL();
		emit layersToShowChanged(value);
	}		
}

void GLDetailView::setTwoDimension(bool value){
	if(value != m_twoDimension){
		m_twoDimension = value;
		m_threeDimension = m_twoDimension ? false : true;
		updateGL();
		emit twoDimensionChanged(value);
	}
}

void GLDetailView::setThreeDimension(bool value){
	if(value != m_threeDimension){
		m_threeDimension = value;
		m_twoDimension = m_threeDimension ? false : true;		
		updateGL();
		emit threeDimensionChanged(value);
	}
}

void GLDetailView::setThreeDCut(int value){
	if(value != m_cut){
		m_cut = value;
		qDebug()<<m_cut;
		updateGL();
		emit threeDCutChanged(value);
	}
}

void GLDetailView::initializeGL()
{
    // Light values and coordinates
    GLfloat  ambientLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat  diffuseLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat lightPos[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat  specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
    //GLfloat  specref[] = { 0.0f, 0.0f, 1.0f, 1.0f };
    
    //buildList();
    glEnable(GL_DEPTH_TEST);	// Hidden surface removal
    glEnable(GL_LIGHTING);

    // Setup and enable light 0
    glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT0,GL_SPECULAR, specular);    
    glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    glEnable(GL_LIGHT0);

    // Enable color tracking
    glEnable(GL_COLOR_MATERIAL);
	
    // Set Material properties to follow glColor values
    glColorMaterial(GL_FRONT, GL_SPECULAR); //AMBIENT_AND_DIFFUSE);
    //glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    // All materials hereafter have full specular reflectivity
    // with a high shine
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
    //glMateriali(GL_FRONT, GL_SHININESS, 256);
    
    // Light blue background
    glClearColor(1.0f, 1.0f, 1.0f, 1.0f );
    // Black background
    //glClearColor(1.0f, 1.0f, 1.0f, 1.0f );

    // Set drawing color to green
    glColor3f(0.0f, 1.0f, 0.0f);
	
  //glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
  //glShadeModel(GL_FLAT);
  //glEnable(GL_DEPTH_TEST);
  //glEnable(GL_CULL_FACE);
}

void GLDetailView::paintGL(){
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glLoadIdentity();
  glPushMatrix();
  
  glTranslated(0.0, 0.0, -100.0);
  glRotated(m_xRot / 16.0, 1.0, 0.0, 0.0);
  glRotated(m_yRot / 16.0, 0.0, 1.0, 0.0);
  glRotated(m_zRot / 16.0, 0.0, 0.0, 1.0);
  glScaled(m_scale,m_scale,m_scale);
  
  if(m_threeDimension){
    drawParticleAndNeighbours();
  }
  if(m_twoDimension){
	drawNeighboursCrossView();  
  }
  
  glPopMatrix();
}

void GLDetailView::drawNeighboursCrossView(){
	  DLAElement* pElm;
	  list<DLAElement*> adjList;
	  list<DLAElement*> bgList;
	  list<DLAElement*> fgList;
	  list<DLAElement*>::const_iterator citr;
		 
	  if(m_simuAggregate!=NULL && m_simuContext!=NULL){
		 pElm = m_simuAggregate->getElementById(m_particleId);
		 //glTranslatef(pElm->getZ(), pElm->getY(), pElm->getX());
	  	 m_simuAggregate->getInfluencingNeighbours( pElm->getX()
		 						 , pElm->getY()
								 , pElm->getZ()
								 , m_distThreshold  //m_simuContext->getDiffusionDistanceThreshold()  //1.05f
								 , adjList);

		 for(citr = adjList.begin(); citr!=adjList.end(); citr++){
			//Split the neighbours' list into background (draw first) and foreground lists.
			qDebug() << (*citr)->getElementId() << "-"<< pElm->getYZDistance(*citr) << "-" << "(" << (*citr)->getX() << ", " << (*citr)->getY() << ", " << (*citr)->getZ() << ")";			 
			if((*citr)->getX()>=pElm->getX()){
				fgList.push_back(*citr);
			}
			else{
				bgList.push_back(*citr);
			}
		 }		 
	  }
	  
	  glPushMatrix();
	  GLUquadricObj* pObj;
	  pObj = gluNewQuadric();
	  
	  glPushMatrix();
	  glDisable(GL_LIGHTING);
	  glDisable(GL_DEPTH_TEST);	  
	  glEnable(GL_BLEND);
	  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	  
	  glColor4f(1.0f, 1.0f, 0.0f, 0.5f);
	  for(citr=bgList.begin(); citr!=bgList.end(); citr++){
		glPushMatrix();
		//glTranslated((*citr)->getZ()-pElm->getZ(), (*citr)->getY()-pElm->getY(), (*citr)->getX()-pElm->getX());
		glTranslated((*citr)->getZ()-pElm->getZ(), (*citr)->getY()-pElm->getY(), 0.0f);
		gluDisk(pObj, 0, m_simuContext->getElementHeight()/2.0, 13, 13);
		glPopMatrix();		
	  }
	  
	  for(citr=fgList.begin(); citr!=fgList.end(); citr++){
		glPushMatrix();
		//glTranslated((*citr)->getZ()-pElm->getZ(), (*citr)->getY()-pElm->getY(), (*citr)->getX()-pElm->getX());
		glTranslated((*citr)->getZ()-pElm->getZ(), (*citr)->getY()-pElm->getY(), 0.0f);
		if(pElm == (*citr)){
			glColor4f(0.0f, 1.0f, 0.0f, 0.5f);
		}
		else{
			glColor4f(1.0f, 1.0f, 0.0f, 0.5f);			
		}
		gluDisk(pObj, 0, m_simuContext->getElementHeight()/2.0, 13, 13);
		glPopMatrix();		  
	  }
	  glDisable(GL_BLEND);
	  glEnable(GL_LIGHTING);
	  glEnable(GL_DEPTH_TEST);
	  glPopMatrix();	  
	  gluDeleteQuadric(pObj);

	  glPopMatrix();
}

void GLDetailView::drawParticleAndNeighbours(){
  glEnable(GL_LIGHTING);
  glPushMatrix();
  glRotatef(90.0f, 0.0f, 1.0f, 0.0f );
  if(m_simuAggregate!=NULL && m_simuContext!=NULL){
	 DLAElement* pElm = m_simuAggregate->getElementById(m_particleId);
	 //glTranslatef(pElm->getZ(), pElm->getY(), pElm->getX());
	 list<DLAElement*> adjList;
  	 m_simuAggregate->getInfluencingNeighbours( pElm->getX()
	 						 , pElm->getY()
							 , pElm->getZ()
							 , m_distThreshold  //m_simuContext->getDiffusionDistanceThreshold()  //1.05f
							 , adjList);
	 list<DLAElement*>::const_iterator citr;
	 for(citr = adjList.begin(); citr!=adjList.end(); citr++){
		qDebug() << (*citr)->getElementId() <<"-"<< pElm->getYZDistance(*citr)<<"-"<< "(" << (*citr)->getX() << ", " << (*citr)->getY() << ", " << (*citr)->getZ() << ")";
		glPushMatrix();
 	    glTranslatef((*citr)->getZ()-pElm->getZ(), (*citr)->getY()-pElm->getY(), (*citr)->getX()-pElm->getX());
 	    if(m_cut!=0){
 	      if(pElm==(*citr)){
	        drawAllTropoelastin( (*citr)->getX()
		        			   , pElm->getX()-10.000001f //m_leftPosition
		    	    		   , pElm->getX()+10.000001f //m_rightPosition		    						 
		    		    	   , m_simuContext->getElementWidth()
		    			       , m_simuContext->getElementHeight()/2.0
		    			       , *(m_simuContext->getElementTemplate())
		    		    	   , (*citr)->getOrientation()
		    			       , 1
		    			       );
 	     }
 	     else{
 	       drawAllTropoelastin( (*citr)->getX()
 		        			  , pElm->getX()-10.0f //m_leftPosition
 		    			      , pElm->getX()+10.0f //m_rightPosition		    						 
 		    			      , m_simuContext->getElementWidth()
 		    			      , m_simuContext->getElementHeight()/2.0
 		    			      , *(m_simuContext->getElementTemplate())
 		    			      , (*citr)->getOrientation()
 		    			      , 0
 		    			      ); 		   
 	     }
 	   }
 	   else{		
 	     if(pElm==(*citr)){
		   drawTropoelastin(m_simuContext->getElementWidth(), m_simuContext->getElementHeight()/2.0, *(m_simuContext->getElementTemplate()), (*citr)->getOrientation(), 1);
 	     }
 	     else{
 		   drawTropoelastin(m_simuContext->getElementWidth(), m_simuContext->getElementHeight()/2.0, *(m_simuContext->getElementTemplate()), (*citr)->getOrientation()); 	    	
 	     } 	    
 	   }
	   glPopMatrix();
	 }
  }
  //gluDeleteQuadric(pObj);
  glPopMatrix();
}

void GLDetailView::resizeGL(int w, int h){
  glViewport(0, 0, w, h);
  
  GLfloat fAspect = (GLfloat)w/(GLfloat)h;
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0f, fAspect, 20.0f, 200.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void GLDetailView::setXRotation(int angle){
  normalizeAngle(&angle);
  if(angle!=m_xRot){
	  m_xRot = angle;
	  emit xRotationChanged(angle);
	  updateGL();
  }
}

void GLDetailView::setYRotation(int angle){
  normalizeAngle(&angle);
  if(angle!=m_yRot){
	  m_yRot = angle;
	  emit yRotationChanged(angle);
	  updateGL();
  }
}

void GLDetailView::setZRotation(int angle){
  normalizeAngle(&angle);
  if(angle!=m_zRot){
	  m_zRot = angle;
	  emit zRotationChanged(angle);
	  updateGL();
  }
}

void GLDetailView::mousePressEvent(QMouseEvent* event){
	m_lastPos = event->pos();
}

void GLDetailView::mouseMoveEvent(QMouseEvent* event){
	int dx = event->x() - m_lastPos.x();
	int dy = event->y() - m_lastPos.y();
	
	if(event->buttons() & Qt::LeftButton){
		setXRotation(m_xRot + 8 * dy);
		setYRotation(m_yRot + 8 * dx);
	}
	else if(event->buttons() & Qt::RightButton){
		setXRotation(m_xRot + 8 * dy);
		setZRotation(m_zRot + 8 * dx);
	}
	m_lastPos = event->pos();
}

void GLDetailView::mouseDoubleClickEvent(QMouseEvent* event){
	if(event->buttons() & Qt::LeftButton){
		setScale(m_scale * 2);
	}
	else if(event->buttons() & Qt::RightButton){
		setScale(m_scale * 0.5);
	}
}

void GLDetailView::normalizeAngle(int* angle){
	while(*angle < 0){
		*angle += 360 * 16;
	}
	while(*angle > 360 * 16){
		*angle -= 360 * 16;
	}
}
