#ifndef GLDETAILVIEW_H_
#define GLDETAILVIEW_H_

#include <QGLWidget>
#include <list>

class Aggregate;
class SimuContext;
class DLAElement;
class QGroupBox;

class GLDetailView : public QGLWidget
{
	Q_OBJECT
	
public:
	GLDetailView(SimuContext* pContext, Aggregate* pAgg, QWidget* parent = 0);
	virtual ~GLDetailView();
	QSize   minimumSizeHint() const;
	QSize   sizeHint() const;
	float   scale() const;
	int     particleId() const;
	float   distThreshold() const;
	int     layersToShow() const;
	int     drawingChoice() const;
	
protected:
	void initializeGL();
	void paintGL();
	void resizeGL(int width, int height);
	void mousePressEvent(QMouseEvent* event);
	void mouseMoveEvent(QMouseEvent* event);
	void mouseDoubleClickEvent(QMouseEvent* event);
	void drawParticleAndNeighbours();
	void drawNeighboursCrossView();
	
public slots:
    void setXRotation(int angle);
    void setYRotation(int angle);
    void setZRotation(int angle);
    void setScale(float value);
    void setScale(const QString& str);
    void setParticleId(int value);
    void setParticleId(const QString& str);    
    void setDistThreshold(float value);
    void setDistThreshold(const QString& str);
    void setLayersToShow(int value);
    void setTwoDimension(bool value);
    void setThreeDimension(bool value);
    void setThreeDCut(int value);
    
signals:
    void xRotationChanged(int angle);
    void yRotationChanged(int angle);
    void zRotationChanged(int angle);
    void scaleChanged(float value);
    void scaleChanged(const QString& str);    
    void particleIdChanged(int value);
    void distThresholdChanged(float value);
    void layersToShowChanged(int value);
    void twoDimensionChanged(bool value);
    void threeDimensionChanged(bool value);
    void threeDCutChanged(int value);
    
private:
	void   normalizeAngle(int* angle);
	//QGroupBox* createOptionGroup();
	int    m_xRot;
	int    m_yRot;
	int    m_zRot;	
	QPoint m_lastPos;
	
	float  m_scale;	
	int    m_particleId;	
	float  m_distThreshold;
	int    m_layersToShow;
	bool   m_twoDimension;
	bool   m_threeDimension;
	int   m_cut;
	
	Aggregate*   m_simuAggregate;
	SimuContext* m_simuContext;
	//std::list<DLAElement*> m_paticleList;
	//QGroupBox*   m_optBox;
};

#endif /*GLDETAILVIEW_H_*/
