#include <QtGui>
#include <QtOpenGL>

#include <cmath>
#include <iostream>
#include <vector>
#include "GLViewWidget.h"
#include "DLAElement.h"
#include "SimuContext.h"
#include "CylinderElastin.h"

using namespace std;

float testData[][3] = { {0.0f, 0.0f, 0.0f}
                      , {2.0f, 2.0f, 2.0f}
                      , {-2.0f, -2.0f, -2.0f}
                      };

GLViewWidget::GLViewWidget(Aggregate * agg, QWidget *parent)
    : QGLWidget(parent), aggregate(agg), m_realtime(false), m_activeElm(NULL){
	object = 0;
    
    xRot = 0; 
    yRot = 0; 
    zRot = 0;
    
    xMov = 0.0f;
    yMov = 0.0f;
    zMov = -100.0f;
    
    viewScale = 1.0f;
    x = 50.0f;
    y = 50.0f;
    z = 50.0f;
}

GLViewWidget::~GLViewWidget()
{
    makeCurrent();
}

void GLViewWidget::initializeGL()
{
    // Light values and coordinates
    GLfloat  ambientLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat  diffuseLight[] = { 0.1f, 0.2f, 0.1f, 1.0f };
    GLfloat lightPos[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat lightPos1[] = { 0.0f, -20.0f, 0.0f, 1.0f };
    GLfloat lightPos2[] = { 0.0f, 20.0f, 0.0f, 1.0f };    
    GLfloat  specular[] = { 0.5f, 0.5f, 0.5f, 1.0f };
    glEnable(GL_DEPTH_TEST);	// Hidden surface removal
    //glShadeModel(GL_SMOOTH);
    glEnable(GL_LIGHTING);

    // Setup and enable light 0
    //glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
    //glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
    //glLightfv(GL_LIGHT0,GL_SPECULAR, specular);    
    //glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    //glEnable(GL_LIGHT0);

    //glLightfv(GL_LIGHT1,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT1,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT1,GL_SPECULAR, specular);    
    glLightfv(GL_LIGHT1,GL_POSITION,lightPos1);
    glEnable(GL_LIGHT1);

    //glLightfv(GL_LIGHT2,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT2,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT2,GL_SPECULAR, specular);    
    glLightfv(GL_LIGHT2,GL_POSITION,lightPos2);
    glEnable(GL_LIGHT2);
    

    // Enable color tracking
    glEnable(GL_COLOR_MATERIAL);
	
    // Set Material properties to follow glColor values
    glColorMaterial(GL_FRONT_AND_BACK, GL_SPECULAR); //AMBIENT_AND_DIFFUSE);
   
    // Light blue background
    glClearColor(1.0f, 1.0f, 1.0f, 1.0f );

    // Set drawing color to green
    glColor3f(0.0f, 1.0f, 0.0f);
}

void GLViewWidget::paintGL()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
  // Save matrix state and do the rotation
  glPushMatrix();
  glTranslatef(xMov, yMov, zMov);
  glRotatef(xRot, 1.0f, 0.0f, 0.0f);
  glRotatef(yRot, 0.0f, 1.0f, 0.0f);
  glRotatef(zRot, 0.0f, 0.0f, 1.0f);  
  glScalef(viewScale, viewScale, viewScale);
  buildList();
  glPopMatrix();
}

void GLViewWidget::resizeGL(int w, int h)
{
  GLfloat fAspect;
  if(h == 0){
    h = 1;
  }

  // Set Viewport to window dimensions
  glViewport(0, 0, w, h);
  fAspect = (GLfloat)w/(GLfloat)h;
  
  // Reset coordinate system
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0f, fAspect, 20.0f, 200.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();  
}


void GLViewWidget::timerEvent(QTimerEvent *){
  xRot += 16;
  glRotatef(xRot, 1.0f, 0.0f, 0.0f);	
  updateGL();
}

QSize GLViewWidget::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize GLViewWidget::sizeHint() const
{
    return QSize(400, 400);
}

void GLViewWidget::setXRotation(int angle)
{
  xRot += (angle-xRot);
  updateGL();
}

void GLViewWidget::setYRotation(int angle)
{
  yRot += (angle-yRot);
  updateGL();
}

void GLViewWidget::setZRotation(int angle)
{
  zRot += (angle-zRot);
  updateGL();
}

void GLViewWidget::setXMovement(float offset)
{
  xMov = offset;
  updateGL();
}

void GLViewWidget::setYMovement(float offset)
{
  yMov = offset;
  updateGL();
}

void GLViewWidget::setZMovement(float offset)
{
  zMov = offset;
  updateGL();
}

void GLViewWidget::setScale(float scale){
  viewScale = scale;
  updateGL();
}
/*
void GLViewWidget::mousePressEvent(QMouseEvent *event)
{
    lastPos = event->pos();
}

void GLViewWidget::mouseDoubleClickEvent(QMouseEvent *event)
{
    if(event->button() == Qt::LeftButton){
      viewScale += 0.1;
    }
    if(event->button() == Qt::RightButton){
      if(viewScale > 0.1){
        viewScale -= 0.1;
      }
    }
   updateGL();
}

void GLViewWidget::mouseMoveEvent(QMouseEvent *event)
{
    int dx = event->x() - lastPos.x();
    int dy = event->y() - lastPos.y();

    if (event->buttons() & Qt::LeftButton) {
        setXRotation(xRot + 8 * dy);
        setYRotation(yRot + 8 * dx);
    } else if (event->buttons() & Qt::RightButton) {
        setXRotation(xRot + 8 * dy);
        setZRotation(zRot + 8 * dx);
    }
    lastPos = event->pos();
}
*/
void GLViewWidget::setAggregate(Aggregate* agg){
	aggregate = agg;
	//readElastinTemplate();
	updateGL();
}

void GLViewWidget::realtimeEvent(){
	m_realtime = true;
	updateGL();
}

void GLViewWidget::newPosition(float x, float y, float z){
	if(m_activeElm==NULL){
		m_activeElm = new CylinderElastin();
	}
	m_activeElm->setPos(x, y, z);
	//qDebug()<<y;
	updateGL();
}

void GLViewWidget::buildList(){
  SimuContext* context = SimuContext::getInstance();
  float radius = context->getElementHeight()/2.0f;
  if(aggregate!=NULL){
    list<DLAElement*>* elList = aggregate->getElements();
    list<DLAElement*>::iterator itr;
    itr = elList->begin();
    glRotatef(90.0f, 0.0f, 1.0f, 0.0f );

    if(m_realtime){
		//qDebug() << "realtime";
    	float colorScheme[3][3]={ {0.0f, 0.0f, 1.0f}
    	                        , {1.0f, 0.0f, 0.0f}
    							, {1.0f, 1.0f, 0.0f}
    							};
    	float colorScheme1[3][3]={ {0.0f, 1.0f, 0.0f}
    	                        , {1.0f, 1.0f, 0.0f}
    							, {0.0f, 1.0f, 1.0f}
    							};

    	float colorScheme2[3][3]={ {0.5f, 0.5f, 0.0f}
    	                        , {0.0f, 0.5f, 0.5f}
    							, {0.5f, 0.5f, 0.5f}
    							};

    	int numOfElm = aggregate->getNumberOfElements();
    	
    	while(itr != elList->end()){
    		glPushMatrix();
    		glTranslatef((*itr)->getZ(), (*itr)->getY(), (*itr)->getX());
    		float startPos[3];
    		startPos[0]=0.0f;
    		startPos[1]=0.0f;
    		startPos[2]=-context->getElementWidth()/2.0f;
    		if((*itr)->getElementId() < numOfElm-1){
    		drawTropoelastin_1( (*itr)->getOrientation()
    						  , startPos
    						  , *(context->getElementTemplate())
    						  , colorScheme
    						  , radius
    						  );
    		
    		}
    		else{
    		drawTropoelastin_1( (*itr)->getOrientation()
    						  , startPos
    						  , *(context->getElementTemplate())
    						  , colorScheme1
    						  , radius
    						  );
    		}
    		
    		//drawTropoelastin(elastinLength, radius, *(context->getElementTemplate()), (*itr)->getOrientation());
    		glPopMatrix();      
    		itr++;
    	}
    	if(m_activeElm!=NULL){
    		glPushMatrix();
    		glTranslatef(m_activeElm->getZ(), m_activeElm->getY(), m_activeElm->getX());
    		float startPos[3];
    		startPos[0]=0.0f;
    		startPos[1]=0.0f;
    		startPos[2]=-context->getElementWidth()/2.0f;
    		drawTropoelastin_1( m_activeElm->getOrientation()
    						  , startPos
    						  , *(context->getElementTemplate())
    						  , colorScheme2
    						  , radius
    						  );
    		glPopMatrix();
    	}
    }
    
    else{
    	while(itr != elList->end()){
    		glPushMatrix();
    		glTranslatef((*itr)->getZ(), (*itr)->getY(), (*itr)->getX());
    		//cout << (*itr)->getX() << ", " << (*itr)->getY()  << ", " << (*itr)->getZ() << endl;
    		drawTropoelastin(elastinLength, radius, *(context->getElementTemplate()), (*itr)->getOrientation());
    		glPopMatrix();      
    		itr++;
    	}
    }
    
    //glEndList();
  }
}

void GLViewWidget::mouseDoubleClickEvent(QMouseEvent *event){
	if(event->button()==Qt::LeftButton){
		QMessageBox::warning(this, tr("My Application"),
                   tr("The document has been modified.\n"
                      "Do you want to save your changes?"),
                   QMessageBox::Save | QMessageBox::Discard
                   | QMessageBox::Cancel,
                   QMessageBox::Save);
	}
}
