#ifndef GLVIEWWIDGET_H_
#define GLVIEWWIDGET_H_

#include <QGLWidget>

#include "visualutils.h"
#include "Aggregate.h"
#include <vector>

using namespace std;


//void drawTropoelastin(float totalLength, float radius, vector<ElastinTemplate* > elTemplate);	

class GLViewWidget : public QGLWidget
{
    Q_OBJECT

public:
    /*GLViewWidget(QWidget *parent = 0);*/
    GLViewWidget(Aggregate * agg=0, QWidget *parent = 0);
    ~GLViewWidget();

    QSize minimumSizeHint() const;
    QSize sizeHint() const;

public slots:
    void setXRotation(int angle);
    void setYRotation(int angle);
    void setZRotation(int angle);
    
    void setXMovement(float offset);
    void setYMovement(float offset);
    void setZMovement(float offset);
        
    void setScale(float scale);
    void setAggregate(Aggregate* agg);
    void realtimeEvent();
    void newPosition(float, float, float);
//    void addElement(DLAElement* elm);

signals:
    void xRotationChanged(int angle);
    void yRotationChanged(int angle);
    void zRotationChanged(int angle);

protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
    void timerEvent(QTimerEvent *event);
//    void mousePressEvent(QMouseEvent *event);
//    void mouseMoveEvent(QMouseEvent *event);
    void mouseDoubleClickEvent(QMouseEvent *event);    

private:
    void normalizeAngle(int *angle);
    //void readElastinTemplate();
    void buildList();

    GLuint object;
    int 	xRot;
    int 	yRot;
    int 	zRot;
    
    float xMov;
    float yMov;
    float zMov;
        
    float x;
    float y;
    float z;
    float speed;
    float viewScale;
    QPoint lastPos;    
    //vector<ElastinTemplate> elastinVector;
    float  elastinLength;    
    Aggregate* aggregate;
    GLuint elementList;
    
    bool m_realtime;
    DLAElement* m_activeElm;
};

#endif /*GLVIEWWIDGET_H_*/
