#ifndef MAINWINDOW_H_
#define MAINWINDOW_H_

#include <QMainWindow>
#include <QGLWidget>
#include <QTcpSocket>
//#include <QDomDocument>
#include <list>
#include <string>
#include "SimuContext.h"
#include "Aggregate.h"
//#include "SimuModel.h"

using namespace std;

class QAction;
class QMenu;
class RenderThread;
//class QSlider;
//class SimuModel;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow();
    ~MainWindow();
    void initialize(string traceFile);

public slots:

    signals:
    void simuPause();
    void simuContinue();
    void newAggregate(Aggregate* agg);
    void realtimeEvent();
    void newPosition(float, float, float);

protected:
    void closeEvent(QCloseEvent *event);


private slots:
    void newFile();
    void open();
    bool save();
    bool saveAs();
    bool exportImage();
    void about();
    void documentWasModified();
    
    void startVisualization();
    void pauseVisualization();
    void resumeVisualization();
    void stopVisualization();
    
    void viewCrossSection();
    void viewParticleDetails();
    void realtimeVisualization();
    void tcpReady();
    void commTcpError(QAbstractSocket::SocketError error);
    void controlTcpError(QAbstractSocket::SocketError error);
    void commSocketAboutToClose();
    
private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void createDockWindows();
    void readSettings();
    void writeSettings();
    bool maybeSave();
    void loadFile(const QString &fileName);
    bool saveFile(const QString &fileName);
    void setCurrentFile(const QString &fileName);
    QString strippedName(const QString &fullFileName);
    //void initializeGLViewWidget(const QString &fullFileName);


//    SimuModel*      simuModel;

    QString  curFile;

    QMenu 	 *fileMenu;
    QMenu 	 *viewMenu;
    QMenu 	 *simuMenu;
    QMenu 	 *helpMenu;

    QToolBar *fileToolBar;
    QToolBar *simuToolBar;
    QToolBar *viewToolBar;

    QAction  *newAct;
    QAction  *openAct;
    QAction  *saveAct;
    QAction  *saveAsAct;
    QAction *exportAct;
    QAction  *exitAct;
    QAction  *aboutAct;
    QAction  *aboutQtAct;

    QAction *viewRotationAct;
    QAction *viewMovementAct;
    QAction *viewZoomAct;
    QAction *viewCrossAct;
    QAction *viewDetailAct;
    QAction *realtimeVisualAct;
    
    QAction *simuStartAct;
    QAction *simuPauseAct;
    QAction *simuContinueAct;
    QAction *simuStopAct;
    
    //QDomDocument domDocument;
    QGLWidget       *glWidget;
    Aggregate     *aggregate;
    SimuContext   *context;
    QTcpSocket    commSocket;
    QTcpSocket    controlSocket;
    QByteArray    m_commBuffer;  //Buffer used to save communication data from the simulation server
    RenderThread* m_renderThread;
    qint32        m_dataSize;
    //WidgetRotation* rotWidget;
    //WidgetMovement* movWidget;
};

#endif /*MAINWINDOW_H_*/
