#include "visualutils.h"
#include <cstdlib>
#include <cmath>
//#include "common/gltools.h"

using namespace std;

static const int coords[6][4][3] = {
   { { +1, -1, -1 }, { -1, -1, -1 }, { -1, +1, -1 }, { +1, +1, -1 } },
   { { +1, +1, -1 }, { -1, +1, -1 }, { -1, +1, +1 }, { +1, +1, +1 } },
   { { +1, -1, +1 }, { +1, -1, -1 }, { +1, +1, -1 }, { +1, +1, +1 } },
   { { -1, -1, -1 }, { -1, -1, +1 }, { -1, +1, +1 }, { -1, +1, -1 } },
   { { +1, -1, +1 }, { -1, -1, +1 }, { -1, -1, -1 }, { +1, -1, -1 } },
   { { -1, -1, +1 }, { +1, -1, +1 }, { +1, +1, +1 }, { -1, +1, +1 } }
};

static const int normals[6][3] = { {0,  0, -1}
                                 , {0,  1,  0}
                                 , {1,  0,  0}
                                 , {-1, 0,  0}
                                 , {0,  -1, 0}
                                 , {0,  0,  1}
                                 };

static const float colors[6][3] = { {1.0f, 0.0f, 0.0f}
                                  , {0.5f, 0.5f, 0.0f}
                                  , {0.0f, 1.0f, 0.0f}
                                  , {0.0f, 0.5f, 0.5f}
                                  , {0.0f, 0.0f, 1.0f}
                                  , {0.5f, 0.0f, 0.5f}
                                  };
void makeCubicFrame(float width, float height, float depth)
{
    int i, j;
    for ( i = 0; i < 6; ++i) {
        glBegin(GL_LINE_STRIP);
        //glColor3f(fabs(3.0-i)/6.0f, fabs(1.0-i)/6.0f, fabs(2.0-i)/6.0f);
         
        for (j = 0; j < 4; ++j) {            
            glVertex3d(width * coords[i][j][0], height * coords[i][j][1],
                       depth * coords[i][j][2]);
        }
        glEnd();
    }
}

void makeCubic(float width, float height, float depth)
{
    //GLTVector3 vNormal;	// Storeage for calculated surface normal
    //int k = rand()%6;
    //glColor3f(colors[k][0], colors[k][1], colors[k][2]);
    int i, j;
    for ( i = 0; i < 6; ++i) {
        glBegin(GL_QUADS);
        glColor3f(colors[i][0], colors[i][1], colors[i][2]);
        glNormal3iv(normals[i]); 
        for (j = 0; j < 4; ++j) {            
            glVertex3d(width * coords[i][j][0], height * coords[i][j][1],
                       depth * coords[i][j][2]);
        }
        glEnd();
    }
}

void makeCylinder(float width, float base, float top, int slices){
  float angle;
  float x, y, z;

  // Draw the base circular plane
  glPushMatrix();
  //glTranslatef(-width/2.0f, 0.0f, 0.0f);
  glPolygonMode(GL_BACK, GL_FILL);
  //glFrontFace(GL_CW);

  glColor3f(1.0f, 0.0f, 0.0f);
  glBegin(GL_TRIANGLE_FAN);
  glVertex3f(-width/2.0f, 0.0f, 0.0f);
  for(angle = 0.0f; angle < 2.0f * GL_PI; angle += GL_PI/slices){
    x = -width/2.0f;
    y = base * sin(angle);
    z = base * cos(angle);
    glVertex3f(x, y, z);
  }
  angle = 2.0f * GL_PI;
  x = -width/2.0f;
  y = base * sin(angle);
  z = base * cos(angle);
  glVertex3f(x, y, z);

  glEnd();
  glPopMatrix();

  // Draw the cylinder body
  glPushMatrix();
  //glTranslatef(width/2.0f, 0.0f, 0.0f);
  glBegin(GL_TRIANGLE_STRIP);
  //glPolygonMode(GL_BACK, GL_FILL);
  //glFrontFace(GL_CW);
  glColor3f(0.0f, 0.0f, 1.0f);
  //glVertex3f(0.0f, 0.0f, 0.0f);
  for(angle = 0.0f; angle < 2.0f * GL_PI; angle += GL_PI/slices){
    x = -width/2.0f;
    y = base * sin(angle);
    z = base * cos(angle);
    glVertex3f(x, y, z);
    x = width/2.0f;
    y = top * sin(angle);
    z = top * cos(angle);
    glVertex3f(x, y, z);
  }
  angle = 2.0f * GL_PI;
  x = -width/2.0f;
  y = base * sin(angle);
  z = base * cos(angle);
  glVertex3f(x, y, z);
  x = width/2.0f;
  y = top * sin(angle);
  z = top * cos(angle);
  glVertex3f(x, y, z);

  glEnd();
  glPopMatrix();

  // Draw the top circular plane
  glPushMatrix();
  glColor3f(1.0f, 0.0f, 0.0f);
  glBegin(GL_TRIANGLE_FAN);
  glVertex3f(width/2.0f, 0.0f, 0.0f);
  for(angle = 0.0f; angle < 2.0f * GL_PI; angle += GL_PI/slices){
    x = width/2.0f;
    y = top * sin(angle);
    z = top * cos(angle);
    glVertex3f(x, y, z);
  }
  angle = 2.0f * GL_PI;
  x = width/2.0f;
  y = top * sin(angle);
  z = top * cos(angle);
  glVertex3f(x, y, z);

  glEnd();

  glPopMatrix();
}	

void drawLetter(char letter, float size){
  float pi=3.14159265;  
  float r = (size*0.9f)/2.0f;
  float step = 1.5f * pi / 10.0f;
  //glPushMatrix();
  switch(letter){
  case 'A':
  case 'C':
    {
      glBegin(GL_LINE_STRIP);
      float angle;
      for(angle = 0.25f * pi; angle < 1.75 * pi; angle += step ){
       glVertex3f(r*cos(angle), r*sin(angle), 0.0f);
      }
      glVertex3f(r*cos(angle), r*sin(angle), 0.0f);
      glEnd();
      break;
    }
  case 'H':
    {
      glBegin(GL_LINES);
      glVertex3f(-r, r, 0.0f);
      glVertex3f(-r, -r, 0.0f);
      glVertex3f(-r, 0.0f, 0.0f);
      glVertex3f(r, 0.0f, 0.0f);
      glVertex3f(r, r, 0.0f);
      glVertex3f(r, -r, 0.0f);
      glEnd();
      break;
    }
  case 'N':
    {
      glBegin(GL_LINES);
      glVertex3f(-r, r, 0.0f);
      glVertex3f(-r, -r, 0.0f);
      glVertex3f(-r, r, 0.0f);
      glVertex3f(r, -r, 0.0f);
      glVertex3f(r, r, 0.0f);
      glVertex3f(r, -r, 0.0f);
      glEnd();
      break;
    }
  case 'O':
    {
      glBegin(GL_LINE_STRIP);
      float angle;
      for(angle = 0.0f; angle < 2.0 * pi; angle += step ){
       glVertex3f(r*cos(angle), r*sin(angle), 0.0f);
      }
      glVertex3f(r*cos(angle), r*sin(angle), 0.0f);
      glEnd();
      break;
    }
  case 'R':
    {
      float sr = r * 0.5f;   
      glBegin(GL_LINE_STRIP);
      float angle;
      for(angle = 0.0f * pi; angle < 2.0f * pi; angle += step ){
       glVertex3f(sr*cos(angle), sr*sin(angle)+sr, 0.0f);
      }
      glVertex3f(sr*cos(angle), sr*sin(angle)+sr, 0.0f);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-sr, r, 0.0f);
      glVertex3f(-sr, -r, 0.0f);
      glVertex3f(-sr, 0, 0.0f);
      glVertex3f(sr, -r, 0.0f);
      glEnd();
      break;
    }

  default:
  break;
  }
}

void drawAminoAcid(int size,char /*name*/){
  GLUquadricObj* pObj;
  glPushMatrix();   
  pObj = gluNewQuadric();

  glColor3f(0.01f, 0.05f, 0.9f); // Draw the alpha carbon atom
  gluSphere(pObj, size, 26, 26);

  glPushMatrix();
  glColor3f(0.2f, 0.2f, 0.5f); //draw the R group
  gluCylinder(pObj, size/4.0f, size/4.0f, size*3, 6, 6);
  glColor3f(0.5f, 0.5f, 0.1f); 
  glTranslatef(0.0f, 0.0f, size*3.0f);
  gluSphere(pObj, size, 26, 26);
  glPopMatrix();

  glPushMatrix();
  glColor3f(0.2f, 0.2f, 0.5f);
  glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
  gluCylinder(pObj, size/4.0f, size/4.0f, size*3, 6, 6);
  glPopMatrix();
  glPushMatrix();
  glColor3f(0.0f, 1.0f, 0.0f); 
  glTranslatef(size*3.0f, 0.0f, 0.0f);
  gluSphere(pObj, size, 26, 26);
  glPopMatrix();

  glPushMatrix();
  glColor3f(0.2f, 0.2f, 0.5f);
  glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
  gluCylinder(pObj, size/4.0f, size/4.0f, size*3, 6, 6);
  glPopMatrix();
  glPushMatrix();
  glColor3f(0.0f, 1.0f, 1.0f); 
  glTranslatef(0.0f, 0.0f, -size*3.0f);
  gluSphere(pObj, size, 26, 26);
  glPopMatrix();

  glPushMatrix();
  glColor3f(0.2f, 0.2f, 0.5f);
  glRotatef(270.0f, 0.0f, 1.0f, 0.0f);
  gluCylinder(pObj, size/4.0f, size/4.0f, size*3, 6, 6);
  glPopMatrix();
  glPushMatrix();
  glColor3f(1.0f, 1.0f, 0.0f); 
  glTranslatef(-size*3.0f, 0.0f, 0.0f);
  gluSphere(pObj, size, 26, 26);

  glPopMatrix();
 
  gluDeleteQuadric(pObj);  
  glPopMatrix();
}

//void drawTropoelastin(float totalLength, float radius, vector<ElastinTemplate > elTemplate, int orientation){
void drawTropoelastin(float totalLength, float radius, const std::vector<ElementTemplate >& elTemplate, int orientation, int colorChoice){
 
  glPushMatrix();
  glTranslatef(0.0f, 0.0f, -totalLength/2.0f);
  
  vector<ElementTemplate>::const_iterator itr;
  vector<ElementTemplate>::const_reverse_iterator ritr;
  
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  gluQuadricDrawStyle(pObj, GLU_FILL); //LINE);
  gluQuadricNormals(pObj, GLU_FLAT); //SMOOTH);  	
  if(orientation==0){
    itr = elTemplate.begin();
    glColor3f(1.0f, 1.0f, 0.1f);
    if((*itr).exonType()=='c'){
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }
    else{
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }
    while(itr!=elTemplate.end()){
      if((*itr).exonType()=='c'){
   	    glColor3f(0.75f, 0.0f, 0.1f);
        gluCylinder(pObj, radius, radius, (*itr).simuLength(), 26, 26);
      }
      else{
  	glColor3f(0.1f, 0.0f, 0.75f);
        gluCylinder(pObj, radius, radius, (*itr).simuLength(), 26, 26);
      }  	

      glTranslatef(0.0f, 0.0f, (*itr).simuLength());  	
      itr++;
    }
    itr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    if((*itr).exonType()=='c'){
      //gluDisk(pObj, 0.0f, radius/2.0f, 26, 26);
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }
    else{
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }

  }
  else{
    ritr = elTemplate.rbegin();
    glColor3f(1.0f, 1.0f, 0.1f);
    if((*ritr).exonType()=='c'){
      //gluDisk(pObj, 0.0f, radius/2.0f, 26, 26);
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }
    else{
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }

    while(ritr!=elTemplate.rend()){
      if((*ritr).exonType()=='c'){
    	 glColor3f(0.75f, 0.0f, 0.1f);
    	 gluCylinder(pObj, radius, radius, (*ritr).simuLength(), 26, 26);
      }
      else{
    	  glColor3f(0.1f, 0.0f, 0.75f);
    	  gluCylinder(pObj, radius, radius, (*ritr).simuLength(), 26, 26);
      } 
      glTranslatef(0.0f, 0.0f, (*ritr).simuLength());  	
      ritr++;
    }

    ritr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    if((*ritr).exonType()=='c'){
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }
    else{
      gluDisk(pObj, 0.0f, radius, 26, 26);
    }
  }
  glPopMatrix();
}

//void drawTropoelastin(float totalLength, float radius, vector<ElastinTemplate > elTemplate, int orientation){
void drawRightPartTropoelastin( float curPosition
							, float lowerBoundary
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int orientation
							, int colorChoice){
 
  glPushMatrix();  
  float offset = lowerBoundary - curPosition;
  glTranslatef(0.0f, 0.0f, offset);
  
  float curOffset = -totalLength/2.0f;
  
  vector<ElementTemplate>::const_iterator itr;
  vector<ElementTemplate>::const_reverse_iterator ritr;
  
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  gluQuadricDrawStyle(pObj, GLU_FILL); //LINE);
  gluQuadricNormals(pObj, GLU_FLAT); //SMOOTH);  	
  if(orientation==0){
    itr = elTemplate.begin();
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    
    gluDisk(pObj, 0.0f, radius, 26, 26);
    while(itr!=elTemplate.end()){
      if(curOffset >= offset ){
    	  if((*itr).exonType()=='c'){
    	  	glColor3f(0.75f, 0.0f, 0.1f);
    	  	gluCylinder(pObj, radius, radius, (*itr).simuLength(), 26, 26);
    	  }
    	  else{
    		  glColor3f(0.1f, 0.0f, 0.75f);
    		  gluCylinder(pObj, radius, radius, (*itr).simuLength(), 26, 26);
    	  }  	

    	  glTranslatef(0.0f, 0.0f, (*itr).simuLength());
    	  curOffset+=(*itr).simuLength();
      }
      else{ //curOffset < offset
    	  if((curOffset+(*itr).simuLength()) > offset){
    		  float diff = fabs(curOffset-offset);
        	  if((*itr).exonType()=='c'){
        	  	glColor3f(0.75f, 0.0f, 0.1f);
        	  	gluCylinder(pObj, radius, radius, (*itr).simuLength()-diff, 26, 26);
        	  }
        	  else{
        		  glColor3f(0.1f, 0.0f, 0.75f);
        		  gluCylinder(pObj, radius, radius, (*itr).simuLength()-diff, 26, 26);
        	  } 
        	  glTranslatef(0.0f, 0.0f, (*itr).simuLength()-diff);
        	  curOffset+=(*itr).simuLength();    		  
    	  }
    	  else{
        	  curOffset+=(*itr).simuLength();    		  
    	  }    	  
      }
      itr++;
    }
    itr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    gluDisk(pObj, 0.0f, radius, 26, 26);
  }
  else{
    ritr = elTemplate.rbegin();
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    
    gluDisk(pObj, 0.0f, radius, 26, 26);
    gluDisk(pObj, 0.0f, radius, 26, 26);

    while(ritr!=elTemplate.rend()){
    	if(curOffset >= offset ){
    		if((*ritr).exonType()=='c'){
    			glColor3f(0.75f, 0.0f, 0.1f);
    			gluCylinder(pObj, radius, radius, (*ritr).simuLength(), 26, 26);
    		}
    		else{
    			glColor3f(0.1f, 0.0f, 0.75f);
    			gluCylinder(pObj, radius, radius, (*ritr).simuLength(), 26, 26);
    		} 
    		glTranslatef(0.0f, 0.0f, (*ritr).simuLength());
    		curOffset +=  (*ritr).simuLength();
    	}
    	else{
      	  if((curOffset+(*ritr).simuLength()) > offset){
      		  float diff = fabs(curOffset-offset);
          	  if((*ritr).exonType()=='c'){
          	  	glColor3f(0.75f, 0.0f, 0.1f);
          	  	gluCylinder(pObj, radius, radius, (*ritr).simuLength()-diff, 26, 26);
          	  }
          	  else{
          		  glColor3f(0.1f, 0.0f, 0.75f);
          		  gluCylinder(pObj, radius, radius, (*ritr).simuLength()-diff, 26, 26);
          	  } 
          	  glTranslatef(0.0f, 0.0f, (*ritr).simuLength()-diff);
          	  curOffset+=(*ritr).simuLength();    		  
      	  }
      	  else{
          	  curOffset+=(*ritr).simuLength();    		  
      	  }    		
    	}
    	ritr++;
    }
    ritr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    
    gluDisk(pObj, 0.0f, radius, 26, 26);
  }
  glPopMatrix();
}

//void drawTropoelastin(float totalLength, float radius, vector<ElastinTemplate > elTemplate, int orientation){
void drawLeftPartTropoelastin( float curPosition
							, float upperBoundary
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int orientation
							, int colorChoice){
 
  glPushMatrix();  
  float offset = upperBoundary - curPosition;
  if(offset>totalLength/2.0f){
	  offset=totalLength/2.0f;
  }
  
  float curOffset = -totalLength/2.0f;
  glTranslatef(0.0f, 0.0f, curOffset);
  vector<ElementTemplate>::const_iterator itr;
  vector<ElementTemplate>::const_reverse_iterator ritr;
  
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  gluQuadricDrawStyle(pObj, GLU_FILL); //LINE);
  gluQuadricNormals(pObj, GLU_FLAT); //SMOOTH);  	
  if(orientation==0){
    itr = elTemplate.begin();
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    
    gluDisk(pObj, 0.0f, radius, 26, 26);
    while(itr!=elTemplate.end()){
      if(curOffset >= offset){
    	  break;
      }
      else{ //curOffset < offset
    	  if( (curOffset + (*itr).simuLength()) <= offset ){
    		  drawExon(pObj, (*itr).exonType(), radius, (*itr).simuLength());    	  
    		  glTranslatef(0.0f, 0.0f, (*itr).simuLength());
    		  curOffset+=(*itr).simuLength();
    	  }
    	  else{ // (curOffset + (*itr).simuLength) >  offset)
    		  float diff = fabs(curOffset-offset);
        	  drawExon(pObj, (*itr).exonType(), radius, diff);    		  
        	  glTranslatef(0.0f, 0.0f, diff);
        	  curOffset+=diff;  		  
    	  }
      }
      itr++;
    }
    itr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    gluDisk(pObj, 0.0f, radius, 26, 26);
  }
  else{
    ritr = elTemplate.rbegin();
    //glColor3f(1.0f, 1.0f, 0.1f);
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    
    gluDisk(pObj, 0.0f, radius, 26, 26);

    while(ritr!=elTemplate.rend()){
    	if(curOffset + (*ritr).simuLength() <= offset ){
    		if((*ritr).exonType()=='c'){
    			glColor3f(0.75f, 0.0f, 0.1f);
    			//gluCylinder(pObj, radius/2.0f, radius/2.0f, (*ritr)->length, 6, 6);
    			gluCylinder(pObj, radius, radius, (*ritr).simuLength(), 26, 26);
    		}
    		else{
    			glColor3f(0.1f, 0.0f, 0.75f);
    			gluCylinder(pObj, radius, radius, (*ritr).simuLength(), 26, 26);
    		} 
    		glTranslatef(0.0f, 0.0f, (*ritr).simuLength());
    		curOffset +=  (*ritr).simuLength();
    	}
    	else{
      	  if(curOffset < offset){
      		  float diff = fabs(curOffset-offset);
          	  if((*ritr).exonType()=='c'){
          	  	glColor3f(0.75f, 0.0f, 0.1f);
          	  	gluCylinder(pObj, radius, radius, diff, 26, 26);
          	  }
          	  else{
          		  glColor3f(0.1f, 0.0f, 0.75f);
          		  gluCylinder(pObj, radius, radius, diff, 26, 26);
          	  } 
          	  break;
      	  }
      	  else{
      		  break;
      	  }    		
    	}
    	ritr++;
    }

    ritr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    gluDisk(pObj, 0.0f, radius, 26, 26);
  }
  glPopMatrix();
}

//void drawTropoelastin(float totalLength, float radius, vector<ElastinTemplate > elTemplate, int orientation){
void drawCenterPartTropoelastin( float curPosition
							, float lowerBoundary		
							, float upperBoundary
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int orientation
							, int colorChoice){
 
  glPushMatrix();  
  float leftOffset = lowerBoundary - curPosition;
  float rightOffset = upperBoundary - curPosition;
  
  float curOffset = -totalLength/2.0f;
  
  glTranslatef(0.0f, 0.0f, leftOffset);


  
  vector<ElementTemplate>::const_iterator itr;
  vector<ElementTemplate>::const_reverse_iterator ritr;
  
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  gluQuadricDrawStyle(pObj, GLU_FILL); //LINE);
  gluQuadricNormals(pObj, GLU_FLAT); //SMOOTH);  	
  float diff=0.0f;  
  if(orientation==0){
    itr = elTemplate.begin();
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    
    gluDisk(pObj, 0.0f, radius, 26, 26);

    while(itr!=elTemplate.end()){
      if(curOffset>=rightOffset){ // |   |-----
    	  break;
      }
      else if(curOffset>=leftOffset && (curOffset+(*itr).simuLength()) <=rightOffset){ // | ----- |
		  drawExon(pObj, (*itr).exonType(), radius, (*itr).simuLength());
		  glTranslatef(0.0f, 0.0f, (*itr).simuLength());
		  curOffset+=(*itr).simuLength();    	  
      }
      else if(curOffset>leftOffset && curOffset < rightOffset && (curOffset+(*itr).simuLength()) > rightOffset){ //| ---|--
		  diff = rightOffset-curOffset;
		  drawExon(pObj, (*itr).exonType(), radius, diff);
		  glTranslatef(0.0f, 0.0f, diff);
		  //curOffset+=diff;
		  break;
      }
      else if(curOffset<leftOffset && curOffset+(*itr).simuLength()<leftOffset){ //-----|  |
		  curOffset+=(*itr).simuLength();
      }      
      else if( curOffset<leftOffset
    		  && curOffset+(*itr).simuLength()>leftOffset 
    		  && curOffset+(*itr).simuLength()<rightOffset){ // ---|-- |
    	  diff = leftOffset - curOffset;
		  drawExon(pObj, (*itr).exonType(), radius, (*itr).simuLength()-diff);
		  glTranslatef(0.0f, 0.0f, (*itr).simuLength()-diff);
		  curOffset+=(*itr).simuLength();    	  
      }
      else if(curOffset<leftOffset && curOffset+(*itr).simuLength() > rightOffset){ // -|---|-
    	  diff = rightOffset - leftOffset;
		  drawExon(pObj, (*itr).exonType(), radius, diff);
		  glTranslatef(0.0f, 0.0f, diff);
		  break;
      }
      else{
    	  cerr << "Not included ---------------- center part! \n";
      }
      itr++;
    }
    itr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    gluDisk(pObj, 0.0f, radius, 26, 26);
  }
  else{
    ritr = elTemplate.rbegin();
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }
    gluDisk(pObj, 0.0f, radius, 26, 26);

    while(ritr!=elTemplate.rend()){
        if(curOffset >= leftOffset ){
      	  if(curOffset>=rightOffset){
      		  break;
      	  }
      	  else{
      		  if( (curOffset + (*ritr).simuLength()) <= rightOffset ){
      			  drawExon(pObj, (*ritr).exonType(), radius, (*ritr).simuLength());
      			  glTranslatef(0.0f, 0.0f, (*ritr).simuLength());
      			  curOffset+=(*ritr).simuLength();
      		  }
      		  else{	// if( (curOffset + (*itr).simuLength) > rightOffset )
      			  diff = rightOffset-curOffset;
      			  drawExon(pObj, (*ritr).exonType(), radius, diff);
      			  break;
      		  }
      	  }
        }
        else{ //curOffset < leftOffset
      	  if((curOffset + (*ritr).simuLength()) <= leftOffset){
      		  curOffset += (*ritr).simuLength();  	  
      	  }
      	  else{ //curOffset + (*itr).simuLength > leftOffset
      		  if( (curOffset + (*ritr).simuLength()) <= rightOffset ){
      			  diff = leftOffset-curOffset;
      			  drawExon(pObj, (*ritr).exonType(), radius, (*ritr).simuLength()-diff);
      			  glTranslatef(0.0f, 0.0f, (*ritr).simuLength()-diff);
      			  curOffset+=((*ritr).simuLength()-diff);
      		  }
      		  else if( (curOffset + (*ritr).simuLength()) > rightOffset ){
      			  diff = rightOffset - leftOffset;
      			  drawExon(pObj, (*ritr).exonType(), radius, diff);
      			  break;
      		  }      
      		  else{
      			  cerr << "Error in drawTropoelastinWithBoundary ------------!" << "\n";
      			  break;
      		  }
      	  }
        }
    	ritr++;
    }
    ritr--;
    if(colorChoice==0){
      glColor3f(1.0f, 1.0f, 0.1f);
    }
    else{
      glColor3f(0.0f, 1.0f, 0.0f);    	
    }    
    gluDisk(pObj, 0.0f, radius, 26, 26);
  }
  glPopMatrix();
}


void drawAllTropoelastin( float curPosition
							, float lowerBoundary		
							, float upperBoundary 
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int orientation
							, int colorChoice){
	if(upperBoundary <= lowerBoundary){
		cerr << "Upperboundary is smaller than lowerboundary in drawPartialTropoelastion -------------!" << "\n";
		return;
	}
	float halfLength = totalLength/2.0f;
	if(curPosition-halfLength < lowerBoundary && curPosition+halfLength < lowerBoundary ){ // -----|   |
		//Do nothing
	}
	else if(curPosition-halfLength < lowerBoundary
			&& curPosition+halfLength > lowerBoundary
			&& curPosition+halfLength < upperBoundary){  // ---|--   |
		drawRightPartTropoelastin(curPosition, lowerBoundary, totalLength, radius, elTemplate, orientation, colorChoice);		
	}
	else if( curPosition-halfLength >= lowerBoundary && curPosition+halfLength <= upperBoundary ){ // | ----- |  
		drawTropoelastin(totalLength, radius, elTemplate, orientation, colorChoice);
	}
	else if(  curPosition-halfLength > lowerBoundary
			&& curPosition-halfLength < upperBoundary
			&& curPosition+halfLength > upperBoundary){ // |  ---|--
		drawLeftPartTropoelastin(curPosition, upperBoundary, totalLength, radius, elTemplate, orientation, colorChoice);
	}
	else if(curPosition-halfLength < lowerBoundary && curPosition+halfLength > upperBoundary){ // -|--|--
		drawCenterPartTropoelastin(curPosition, lowerBoundary, upperBoundary, totalLength, radius, elTemplate, orientation, colorChoice);
	}
	else{
		cerr << "Not included ------------ draw all!" << "\n";
	}
}

void drawExon(GLUquadricObj* pObj, char exonType, float radius, float visualLength){
	if(exonType=='c'){
		glColor3f(0.75f, 0.0f, 0.1f);
		gluCylinder(pObj, radius, radius, visualLength, 26, 26);
	}
	else{
	  glColor3f(0.1f, 0.0f, 0.75f);
	  gluCylinder(pObj, radius, radius, visualLength, 26, 26);
	}	 
}

/**
 * New version of drawTropoelastin
 * The previous version includes all the drawing actions in one function.
 * The new version tries to split the previous one and use color as parameter.
 * 
 */
void drawTropoelastin_1( int orientation
					   , const float (&startPos)[3]
		               , const std::vector<ElementTemplate >& elTemplate
		               , const float (&colorScheme)[3][3]
		               , float radius		                      
		               ){
	if(orientation==0){
		drawTropoelastin_normal_1( startPos
								  , elTemplate
								  , colorScheme
								  , radius
								  );
	}
	else if(orientation==1){
		drawTropoelastin_reverse_1( startPos
								   , elTemplate
								   , colorScheme
								   , radius
								   );		
	}
	else{
		std::cerr << "Unknown orientation: " << orientation << "\n";
	}
}

void drawTropoelastin_normal_1( const float (&startPos)[3]
		                      , const std::vector<ElementTemplate >& elTemplate
		                      , const float (&colorScheme)[3][3]
		               		  , float radius		                      
		                      ){ 
  glPushMatrix();
  glTranslatef(startPos[0], startPos[1], startPos[2]); //Start position from negative direction  
  vector<ElementTemplate>::const_iterator itr;
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  gluQuadricDrawStyle(pObj, GLU_FILL); //LINE);
  gluQuadricNormals(pObj, GLU_FLAT); //SMOOTH);
  
  itr = elTemplate.begin();
  glFrontFace(GL_CW);
  glColor3f(colorScheme[2][0], colorScheme[2][1], colorScheme[2][2]);
  gluDisk(pObj, 0.0f, radius, 26, 26);
  glFrontFace(GL_CCW);
  while(itr!=elTemplate.end()){
	  drawExon_1( pObj
	  		    , (*itr).simuLength()
	  		    , (*itr).exonType()
	  			, colorScheme[0]
	  			, colorScheme[1]			   
	  			, radius
	  			);
      glTranslatef(0.0f, 0.0f, (*itr).simuLength());  	
      itr++;
  }
  glColor3f(colorScheme[2][0], colorScheme[2][1], colorScheme[2][2]);
  gluDisk(pObj, 0.0f, radius, 26, 26);
  glPopMatrix();
}

void drawTropoelastin_reverse_1( const float (&startPos)[3]
		                       , const std::vector<ElementTemplate >& elTemplate
		                       , const float (&colorScheme)[3][3]
		               		   , float radius		                      
		                       ){ 
  glPushMatrix();
  glTranslatef(startPos[0], startPos[1], startPos[2]); //Start position from negative direction  
  vector<ElementTemplate>::const_reverse_iterator itr;
  GLUquadricObj* pObj;
  pObj = gluNewQuadric();
  gluQuadricDrawStyle(pObj, GLU_FILL); //LINE);
  gluQuadricNormals(pObj, GLU_FLAT); //SMOOTH);
  
  itr = elTemplate.rbegin();
  glFrontFace(GL_CW);
  glColor3f(colorScheme[2][0], colorScheme[2][1], colorScheme[2][2]);
  gluDisk(pObj, 0.0f, radius, 26, 26);
  glFrontFace(GL_CCW);
  while(itr!=elTemplate.rend()){
	  drawExon_1( pObj
	  		    , (*itr).simuLength()
	  		    , (*itr).exonType()
	  			, colorScheme[0]
	  			, colorScheme[1]			   
	  			, radius
	  			);
      glTranslatef(0.0f, 0.0f, (*itr).simuLength());  	
      itr++;
  }
  glColor3f(colorScheme[2][0], colorScheme[2][1], colorScheme[2][2]);
  gluDisk(pObj, 0.0f, radius, 26, 26);
  glPopMatrix();
}

void drawExon_1( GLUquadricObj* pObj
		       , float visualLength		
			   , char  exonType
			   , const float (&hydroColor)[3]
			   , const float (&crossColor)[3]			   
			   , float radius
			   ){
	if(exonType=='h'){
		glColor3f(hydroColor[0], hydroColor[1], hydroColor[2]);
		gluCylinder(pObj, radius, radius, visualLength, 26, 26);
	}
	else if(exonType=='c'){
	  glColor3f(crossColor[0], crossColor[1], crossColor[2]);
	  gluCylinder(pObj, radius, radius, visualLength, 26, 26);
	}
	else{
		std::cerr << "Unknown exon type: " << exonType << "\n";
	}
}
