#ifndef VISUALUTILS_H
#define VISUALUTILS_H

#include <QtOpenGL>
#include <vector>
#include "utils.h"
#include "SimuContext.h"

void makeCubicFrame(float width, float height, float depth);
void makeCubic(float width, float height, float depth);
void makeCylinder(float base, float top, float width, int slices);
//void drawTropoelastin(float totalLength, float radius, vector<ElastinTemplate > elTemplate, int orientation);
void drawTropoelastin(float totalLength, float radius, const std::vector<ElementTemplate >& elTemplate, int orientation, int colorChoice=0);
void drawRightPartTropoelastin( float curPosition
							, float lowerBoundary
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int   orientation
							, int   colorChoice=0);

void drawLeftPartTropoelastin( float curPosition
							, float upperBoundary
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int   orientation
							, int   colorChoice=0);


void drawExon(GLUquadricObj* pObj, char exonType, float radius, float visualLength);
void drawCenterPartTropoelastin( float curPosition
							, float lowerBoundary		
							, float upperBoundary
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int   orientation
							, int   colorChoice=0);

void drawAllTropoelastin( float curPosition
							, float lowerBoundary		
							, float upperBoundary 
							, float totalLength
							, float radius
							, const std::vector<ElementTemplate >& elTemplate
							, int orientation
							, int colorChoice=0);

void drawAminoAcid(int size=1,char name='-');
void drawLetter(char letter, float size=5.0f);

/**
 * New version of drawTropoelastin
 * The previous version includes all the drawing actions in one function.
 * The new version tries to split the previous one and use color as parameter.
 * 
 */
void drawTropoelastin_1( int orientation
					   , const float (&startPos)[3]
		               , const std::vector<ElementTemplate >& elTemplate
		               , const float (&colorScheme)[3][3]
		               , float radius		                      
		               );

void drawTropoelastin_normal_1( const float (&startPos)[3]
		                      , const std::vector<ElementTemplate >& elTemplate
		                      , const float (&colorScheme)[3][3]
		               		  , float radius		                      
		                      ); 

void drawTropoelastin_reverse_1( const float (&startPos)[3]
		                       , const std::vector<ElementTemplate >& elTemplate
		                       , const float (&colorScheme)[3][3]
		               		   , float radius		                      
		                       ); 

void drawExon_1( GLUquadricObj* pObj
		       , float visualLength		
			   , char  exonType
			   , const float (&hydroColor)[3]
			   , const float (&crossColor)[3]			   
			   , float radius
			   );

#endif
